@@ -8,6 +8,7 @@ import Pretty
 import PPUtil
 
 import AbstractSyntax
+import TokenDef
 
 -- AbstractSyntax.ag imports
 import Data.Set(Set)
@@ -18,113 +19,10 @@ import CommonTypes
 
 -- Patterns.ag imports
 import UU.Scanner.Position(Pos)
-import CommonTypes (Constructor,Name)
+import CommonTypes (ConstructorIdent,Identifier)
 
 import UU.Scanner.Position(Pos)
--- Alternative -------------------------------------------------
-{-
-   visit 0:
-      synthesized attribute:
-         pp                   : PP_Doc
-   alternatives:
-      alternative Alternative:
-         child con            : {Constructor}
-         child children       : Children
-         child rules          : Rules
-         child typeSigs       : TypeSigs
--}
--- cata
-sem_Alternative :: Alternative ->
-                   T_Alternative
-sem_Alternative (Alternative _con _children _rules _typeSigs) =
-    (sem_Alternative_Alternative _con (sem_Children _children) (sem_Rules _rules) (sem_TypeSigs _typeSigs))
--- semantic domain
-newtype T_Alternative = T_Alternative (( PP_Doc))
-data Inh_Alternative = Inh_Alternative {}
-data Syn_Alternative = Syn_Alternative {pp_Syn_Alternative :: PP_Doc}
-wrap_Alternative (T_Alternative sem) (Inh_Alternative ) =
-    (let ( _lhsOpp) =
-             (sem )
-     in  (Syn_Alternative _lhsOpp))
-sem_Alternative_Alternative :: Constructor ->
-                               T_Children ->
-                               T_Rules ->
-                               T_TypeSigs ->
-                               T_Alternative
-sem_Alternative_Alternative con_ (T_Children children_) (T_Rules rules_) (T_TypeSigs typeSigs_) =
-    (T_Alternative (let _lhsOpp :: PP_Doc
-                        _childrenIpp :: PP_Doc
-                        _childrenIppL :: ([PP_Doc])
-                        _rulesIpp :: PP_Doc
-                        _rulesIppL :: ([PP_Doc])
-                        _typeSigsIpp :: PP_Doc
-                        _typeSigsIppL :: ([PP_Doc])
-                        -- "AbstractSyntaxDump.ag"(line 31, column 25)
-                        _lhsOpp =
-                            ppNestInfo ["Alternative","Alternative"] [pp con_] [ppF "children" $ ppVList _childrenIppL,ppF "rules" $ ppVList _rulesIppL,ppF "typeSigs" $ ppVList _typeSigsIppL] []
-                        ( _childrenIpp,_childrenIppL) =
-                            (children_ )
-                        ( _rulesIpp,_rulesIppL) =
-                            (rules_ )
-                        ( _typeSigsIpp,_typeSigsIppL) =
-                            (typeSigs_ )
-                    in  ( _lhsOpp)))
--- Alternatives ------------------------------------------------
-{-
-   visit 0:
-      synthesized attributes:
-         pp                   : PP_Doc
-         ppL                  : [PP_Doc]
-   alternatives:
-      alternative Cons:
-         child hd             : Alternative
-         child tl             : Alternatives
-      alternative Nil:
--}
--- cata
-sem_Alternatives :: Alternatives ->
-                    T_Alternatives
-sem_Alternatives list =
-    (Prelude.foldr sem_Alternatives_Cons sem_Alternatives_Nil (Prelude.map sem_Alternative list))
--- semantic domain
-newtype T_Alternatives = T_Alternatives (( PP_Doc,([PP_Doc])))
-data Inh_Alternatives = Inh_Alternatives {}
-data Syn_Alternatives = Syn_Alternatives {pp_Syn_Alternatives :: PP_Doc,ppL_Syn_Alternatives :: [PP_Doc]}
-wrap_Alternatives (T_Alternatives sem) (Inh_Alternatives ) =
-    (let ( _lhsOpp,_lhsOppL) =
-             (sem )
-     in  (Syn_Alternatives _lhsOpp _lhsOppL))
-sem_Alternatives_Cons :: T_Alternative ->
-                         T_Alternatives ->
-                         T_Alternatives
-sem_Alternatives_Cons (T_Alternative hd_) (T_Alternatives tl_) =
-    (T_Alternatives (let _lhsOppL :: ([PP_Doc])
-                         _lhsOpp :: PP_Doc
-                         _hdIpp :: PP_Doc
-                         _tlIpp :: PP_Doc
-                         _tlIppL :: ([PP_Doc])
-                         -- "AbstractSyntaxDump.ag"(line 70, column 33)
-                         _lhsOppL =
-                             _hdIpp : _tlIppL
-                         -- use rule "AbstractSyntaxDump.ag"(line 16, column 58)
-                         _lhsOpp =
-                             _hdIpp >-< _tlIpp
-                         ( _hdIpp) =
-                             (hd_ )
-                         ( _tlIpp,_tlIppL) =
-                             (tl_ )
-                     in  ( _lhsOpp,_lhsOppL)))
-sem_Alternatives_Nil :: T_Alternatives
-sem_Alternatives_Nil  =
-    (T_Alternatives (let _lhsOppL :: ([PP_Doc])
-                         _lhsOpp :: PP_Doc
-                         -- "AbstractSyntaxDump.ag"(line 71, column 33)
-                         _lhsOppL =
-                             []
-                         -- use rule "AbstractSyntaxDump.ag"(line 16, column 58)
-                         _lhsOpp =
-                             empty
-                     in  ( _lhsOpp,_lhsOppL)))
+import HsToken
 -- Child -------------------------------------------------------
 {-
    visit 0:
@@ -132,7 +30,7 @@ sem_Alternatives_Nil  =
          pp                   : PP_Doc
    alternatives:
       alternative Child:
-         child name           : {Name}
+         child name           : {Identifier}
          child tp             : {Type}
          child inh            : {Attributes}
          child syn            : {Attributes}
@@ -151,7 +49,7 @@ wrap_Child (T_Child sem) (Inh_Child ) =
     (let ( _lhsOpp) =
              (sem )
      in  (Syn_Child _lhsOpp))
-sem_Child_Child :: Name ->
+sem_Child_Child :: Identifier ->
                    Type ->
                    Attributes ->
                    Attributes ->
@@ -159,7 +57,7 @@ sem_Child_Child :: Name ->
                    T_Child
 sem_Child_Child name_ tp_ inh_ syn_ higherOrder_ =
     (T_Child (let _lhsOpp :: PP_Doc
-                  -- "AbstractSyntaxDump.ag"(line 34, column 33)
+                  -- "AbstractSyntaxDump.ag"(line 35, column 33)
                   _lhsOpp =
                       ppNestInfo ["Child","Child"] [pp name_, ppShow tp_] [ppF "inh" $ ppMap inh_, ppF "syn" $ ppMap syn_] []
               in  ( _lhsOpp)))
@@ -197,10 +95,10 @@ sem_Children_Cons (T_Child hd_) (T_Children tl_) =
                      _hdIpp :: PP_Doc
                      _tlIpp :: PP_Doc
                      _tlIppL :: ([PP_Doc])
-                     -- "AbstractSyntaxDump.ag"(line 66, column 33)
+                     -- "AbstractSyntaxDump.ag"(line 67, column 33)
                      _lhsOppL =
                          _hdIpp : _tlIppL
-                     -- use rule "AbstractSyntaxDump.ag"(line 16, column 58)
+                     -- use rule "AbstractSyntaxDump.ag"(line 17, column 58)
                      _lhsOpp =
                          _hdIpp >-< _tlIpp
                      ( _hdIpp) =
@@ -212,10 +110,10 @@ sem_Children_Nil :: T_Children
 sem_Children_Nil  =
     (T_Children (let _lhsOppL :: ([PP_Doc])
                      _lhsOpp :: PP_Doc
-                     -- "AbstractSyntaxDump.ag"(line 67, column 33)
+                     -- "AbstractSyntaxDump.ag"(line 68, column 33)
                      _lhsOppL =
                          []
-                     -- use rule "AbstractSyntaxDump.ag"(line 16, column 58)
+                     -- use rule "AbstractSyntaxDump.ag"(line 17, column 58)
                      _lhsOpp =
                          empty
                  in  ( _lhsOpp,_lhsOppL)))
@@ -227,13 +125,13 @@ sem_Children_Nil  =
    alternatives:
       alternative Expression:
          child pos            : {Pos}
-         child txt            : {String}
+         child tks            : {[HsToken]}
 -}
 -- cata
 sem_Expression :: Expression ->
                   T_Expression
-sem_Expression (Expression _pos _txt) =
-    (sem_Expression_Expression _pos _txt)
+sem_Expression (Expression _pos _tks) =
+    (sem_Expression_Expression _pos _tks)
 -- semantic domain
 newtype T_Expression = T_Expression (( PP_Doc))
 data Inh_Expression = Inh_Expression {}
@@ -243,13 +141,13 @@ wrap_Expression (T_Expression sem) (Inh_Expression ) =
              (sem )
      in  (Syn_Expression _lhsOpp))
 sem_Expression_Expression :: Pos ->
-                             String ->
+                             ([HsToken]) ->
                              T_Expression
-sem_Expression_Expression pos_ txt_ =
+sem_Expression_Expression pos_ tks_ =
     (T_Expression (let _lhsOpp :: PP_Doc
-                       -- "AbstractSyntaxDump.ag"(line 49, column 25)
+                       -- "AbstractSyntaxDump.ag"(line 50, column 25)
                        _lhsOpp =
-                           ppNestInfo ["Expression","Expression"] [ppShow pos_] [ppF "txt" $ text txt_] []
+                           ppNestInfo ["Expression","Expression"] [ppShow pos_] [ppF "txt" $ vlist . showTokens . tokensToStrings $ tks_] []
                    in  ( _lhsOpp)))
 -- Grammar -----------------------------------------------------
 {-
@@ -261,16 +159,18 @@ sem_Expression_Expression pos_ txt_ =
          child typeSyns       : {TypeSyns}
          child useMap         : {UseMap}
          child derivings      : {Derivings}
-         child wrappers       : {Set Nonterminal}
-         child prods          : Productions
+         child wrappers       : {Set NontermIdent}
+         child nonts          : Nonterminals
          child pragmas        : {PragmaMap}
          child manualAttrOrderMap : {AttrOrderMap}
+         child paramMap       : {ParamMap}
+         child contextMap     : {ContextMap}
 -}
 -- cata
 sem_Grammar :: Grammar ->
                T_Grammar
-sem_Grammar (Grammar _typeSyns _useMap _derivings _wrappers _prods _pragmas _manualAttrOrderMap) =
-    (sem_Grammar_Grammar _typeSyns _useMap _derivings _wrappers (sem_Productions _prods) _pragmas _manualAttrOrderMap)
+sem_Grammar (Grammar _typeSyns _useMap _derivings _wrappers _nonts _pragmas _manualAttrOrderMap _paramMap _contextMap) =
+    (sem_Grammar_Grammar _typeSyns _useMap _derivings _wrappers (sem_Nonterminals _nonts) _pragmas _manualAttrOrderMap _paramMap _contextMap)
 -- semantic domain
 newtype T_Grammar = T_Grammar (( PP_Doc))
 data Inh_Grammar = Inh_Grammar {}
@@ -282,27 +182,127 @@ wrap_Grammar (T_Grammar sem) (Inh_Grammar ) =
 sem_Grammar_Grammar :: TypeSyns ->
                        UseMap ->
                        Derivings ->
-                       (Set Nonterminal) ->
-                       T_Productions ->
+                       (Set NontermIdent) ->
+                       T_Nonterminals ->
                        PragmaMap ->
                        AttrOrderMap ->
+                       ParamMap ->
+                       ContextMap ->
                        T_Grammar
-sem_Grammar_Grammar typeSyns_ useMap_ derivings_ wrappers_ (T_Productions prods_) pragmas_ manualAttrOrderMap_ =
+sem_Grammar_Grammar typeSyns_ useMap_ derivings_ wrappers_ (T_Nonterminals nonts_) pragmas_ manualAttrOrderMap_ paramMap_ contextMap_ =
     (T_Grammar (let _lhsOpp :: PP_Doc
-                    _prodsIpp :: PP_Doc
-                    _prodsIppL :: ([PP_Doc])
-                    -- "AbstractSyntaxDump.ag"(line 19, column 25)
+                    _nontsIpp :: PP_Doc
+                    _nontsIppL :: ([PP_Doc])
+                    -- "AbstractSyntaxDump.ag"(line 20, column 25)
                     _lhsOpp =
                         ppNestInfo ["Grammar","Grammar"] []
                            [ ppF "typeSyns" $ ppAssocL typeSyns_
                            , ppF "useMap" $ ppMap $ Map.map ppMap $ useMap_
                            , ppF "derivings" $ ppMap $ derivings_
                            , ppF "wrappers" $ ppShow $ wrappers_
-                           , ppF "prods" $ ppVList _prodsIppL
+                           , ppF "nonts" $ ppVList _nontsIppL
                            ] []
-                    ( _prodsIpp,_prodsIppL) =
-                        (prods_ )
+                    ( _nontsIpp,_nontsIppL) =
+                        (nonts_ )
                 in  ( _lhsOpp)))
+-- Nonterminal -------------------------------------------------
+{-
+   visit 0:
+      synthesized attribute:
+         pp                   : PP_Doc
+   alternatives:
+      alternative Nonterminal:
+         child nt             : {NontermIdent}
+         child params         : {[Identifier]}
+         child inh            : {Attributes}
+         child syn            : {Attributes}
+         child prods          : Productions
+-}
+-- cata
+sem_Nonterminal :: Nonterminal ->
+                   T_Nonterminal
+sem_Nonterminal (Nonterminal _nt _params _inh _syn _prods) =
+    (sem_Nonterminal_Nonterminal _nt _params _inh _syn (sem_Productions _prods))
+-- semantic domain
+newtype T_Nonterminal = T_Nonterminal (( PP_Doc))
+data Inh_Nonterminal = Inh_Nonterminal {}
+data Syn_Nonterminal = Syn_Nonterminal {pp_Syn_Nonterminal :: PP_Doc}
+wrap_Nonterminal (T_Nonterminal sem) (Inh_Nonterminal ) =
+    (let ( _lhsOpp) =
+             (sem )
+     in  (Syn_Nonterminal _lhsOpp))
+sem_Nonterminal_Nonterminal :: NontermIdent ->
+                               ([Identifier]) ->
+                               Attributes ->
+                               Attributes ->
+                               T_Productions ->
+                               T_Nonterminal
+sem_Nonterminal_Nonterminal nt_ params_ inh_ syn_ (T_Productions prods_) =
+    (T_Nonterminal (let _lhsOpp :: PP_Doc
+                        _prodsIpp :: PP_Doc
+                        _prodsIppL :: ([PP_Doc])
+                        -- "AbstractSyntaxDump.ag"(line 29, column 25)
+                        _lhsOpp =
+                            ppNestInfo ["Nonterminal","Nonterminal"] (pp nt_ : map pp params_) [ppF "inh" $ ppMap inh_, ppF "syn" $ ppMap syn_, ppF "prods" $ ppVList _prodsIppL] []
+                        ( _prodsIpp,_prodsIppL) =
+                            (prods_ )
+                    in  ( _lhsOpp)))
+-- Nonterminals ------------------------------------------------
+{-
+   visit 0:
+      synthesized attributes:
+         pp                   : PP_Doc
+         ppL                  : [PP_Doc]
+   alternatives:
+      alternative Cons:
+         child hd             : Nonterminal
+         child tl             : Nonterminals
+      alternative Nil:
+-}
+-- cata
+sem_Nonterminals :: Nonterminals ->
+                    T_Nonterminals
+sem_Nonterminals list =
+    (Prelude.foldr sem_Nonterminals_Cons sem_Nonterminals_Nil (Prelude.map sem_Nonterminal list))
+-- semantic domain
+newtype T_Nonterminals = T_Nonterminals (( PP_Doc,([PP_Doc])))
+data Inh_Nonterminals = Inh_Nonterminals {}
+data Syn_Nonterminals = Syn_Nonterminals {pp_Syn_Nonterminals :: PP_Doc,ppL_Syn_Nonterminals :: [PP_Doc]}
+wrap_Nonterminals (T_Nonterminals sem) (Inh_Nonterminals ) =
+    (let ( _lhsOpp,_lhsOppL) =
+             (sem )
+     in  (Syn_Nonterminals _lhsOpp _lhsOppL))
+sem_Nonterminals_Cons :: T_Nonterminal ->
+                         T_Nonterminals ->
+                         T_Nonterminals
+sem_Nonterminals_Cons (T_Nonterminal hd_) (T_Nonterminals tl_) =
+    (T_Nonterminals (let _lhsOppL :: ([PP_Doc])
+                         _lhsOpp :: PP_Doc
+                         _hdIpp :: PP_Doc
+                         _tlIpp :: PP_Doc
+                         _tlIppL :: ([PP_Doc])
+                         -- "AbstractSyntaxDump.ag"(line 75, column 33)
+                         _lhsOppL =
+                             _hdIpp : _tlIppL
+                         -- use rule "AbstractSyntaxDump.ag"(line 17, column 58)
+                         _lhsOpp =
+                             _hdIpp >-< _tlIpp
+                         ( _hdIpp) =
+                             (hd_ )
+                         ( _tlIpp,_tlIppL) =
+                             (tl_ )
+                     in  ( _lhsOpp,_lhsOppL)))
+sem_Nonterminals_Nil :: T_Nonterminals
+sem_Nonterminals_Nil  =
+    (T_Nonterminals (let _lhsOppL :: ([PP_Doc])
+                         _lhsOpp :: PP_Doc
+                         -- "AbstractSyntaxDump.ag"(line 76, column 33)
+                         _lhsOppL =
+                             []
+                         -- use rule "AbstractSyntaxDump.ag"(line 17, column 58)
+                         _lhsOpp =
+                             empty
+                     in  ( _lhsOpp,_lhsOppL)))
 -- Pattern -----------------------------------------------------
 {-
    visit 0:
@@ -311,17 +311,21 @@ sem_Grammar_Grammar typeSyns_ useMap_ derivings_ wrappers_ (T_Productions prods_
          pp                   : PP_Doc
    alternatives:
       alternative Alias:
-         child field          : {Name}
-         child attr           : {Name}
+         child field          : {Identifier}
+         child attr           : {Identifier}
          child pat            : Pattern
          child parts          : Patterns
          visit 0:
             local copy        : _
       alternative Constr:
-         child name           : {Constructor}
+         child name           : {ConstructorIdent}
          child pats           : Patterns
          visit 0:
             local copy        : _
+      alternative Irrefutable:
+         child pat            : Pattern
+         visit 0:
+            local copy        : _
       alternative Product:
          child pos            : {Pos}
          child pats           : Patterns
@@ -339,6 +343,8 @@ sem_Pattern (Alias _field _attr _pat _parts) =
     (sem_Pattern_Alias _field _attr (sem_Pattern _pat) (sem_Patterns _parts))
 sem_Pattern (Constr _name _pats) =
     (sem_Pattern_Constr _name (sem_Patterns _pats))
+sem_Pattern (Irrefutable _pat) =
+    (sem_Pattern_Irrefutable (sem_Pattern _pat))
 sem_Pattern (Product _pos _pats) =
     (sem_Pattern_Product _pos (sem_Patterns _pats))
 sem_Pattern (Underscore _pos) =
@@ -351,8 +357,8 @@ wrap_Pattern (T_Pattern sem) (Inh_Pattern ) =
     (let ( _lhsOcopy,_lhsOpp) =
              (sem )
      in  (Syn_Pattern _lhsOcopy _lhsOpp))
-sem_Pattern_Alias :: Name ->
-                     Name ->
+sem_Pattern_Alias :: Identifier ->
+                     Identifier ->
                      T_Pattern ->
                      T_Patterns ->
                      T_Pattern
@@ -364,7 +370,7 @@ sem_Pattern_Alias field_ attr_ (T_Pattern pat_) (T_Patterns parts_) =
                     _partsIcopy :: Patterns
                     _partsIpp :: PP_Doc
                     _partsIppL :: ([PP_Doc])
-                    -- "AbstractSyntaxDump.ag"(line 45, column 33)
+                    -- "AbstractSyntaxDump.ag"(line 46, column 33)
                     _lhsOpp =
                         ppNestInfo ["Pattern","Alias"] [pp field_, pp attr_] [ppF "pat" $ _patIpp] []
                     -- self rule
@@ -378,7 +384,7 @@ sem_Pattern_Alias field_ attr_ (T_Pattern pat_) (T_Patterns parts_) =
                     ( _partsIcopy,_partsIpp,_partsIppL) =
                         (parts_ )
                 in  ( _lhsOcopy,_lhsOpp)))
-sem_Pattern_Constr :: Constructor ->
+sem_Pattern_Constr :: ConstructorIdent ->
                       T_Patterns ->
                       T_Pattern
 sem_Pattern_Constr name_ (T_Patterns pats_) =
@@ -387,7 +393,7 @@ sem_Pattern_Constr name_ (T_Patterns pats_) =
                     _patsIcopy :: Patterns
                     _patsIpp :: PP_Doc
                     _patsIppL :: ([PP_Doc])
-                    -- "AbstractSyntaxDump.ag"(line 43, column 33)
+                    -- "AbstractSyntaxDump.ag"(line 44, column 33)
                     _lhsOpp =
                         ppNestInfo ["Pattern","Constr"] [pp name_] [ppF "pats" $ ppVList _patsIppL] []
                     -- self rule
@@ -399,6 +405,25 @@ sem_Pattern_Constr name_ (T_Patterns pats_) =
                     ( _patsIcopy,_patsIpp,_patsIppL) =
                         (pats_ )
                 in  ( _lhsOcopy,_lhsOpp)))
+sem_Pattern_Irrefutable :: T_Pattern ->
+                           T_Pattern
+sem_Pattern_Irrefutable (T_Pattern pat_) =
+    (T_Pattern (let _lhsOpp :: PP_Doc
+                    _lhsOcopy :: Pattern
+                    _patIcopy :: Pattern
+                    _patIpp :: PP_Doc
+                    -- use rule "AbstractSyntaxDump.ag"(line 17, column 58)
+                    _lhsOpp =
+                        _patIpp
+                    -- self rule
+                    _copy =
+                        Irrefutable _patIcopy
+                    -- self rule
+                    _lhsOcopy =
+                        _copy
+                    ( _patIcopy,_patIpp) =
+                        (pat_ )
+                in  ( _lhsOcopy,_lhsOpp)))
 sem_Pattern_Product :: Pos ->
                        T_Patterns ->
                        T_Pattern
@@ -408,7 +433,7 @@ sem_Pattern_Product pos_ (T_Patterns pats_) =
                     _patsIcopy :: Patterns
                     _patsIpp :: PP_Doc
                     _patsIppL :: ([PP_Doc])
-                    -- "AbstractSyntaxDump.ag"(line 44, column 33)
+                    -- "AbstractSyntaxDump.ag"(line 45, column 33)
                     _lhsOpp =
                         ppNestInfo ["Pattern","Product"] [ppShow pos_] [ppF "pats" $ ppVList _patsIppL] []
                     -- self rule
@@ -425,7 +450,7 @@ sem_Pattern_Underscore :: Pos ->
 sem_Pattern_Underscore pos_ =
     (T_Pattern (let _lhsOpp :: PP_Doc
                     _lhsOcopy :: Pattern
-                    -- "AbstractSyntaxDump.ag"(line 46, column 25)
+                    -- "AbstractSyntaxDump.ag"(line 47, column 25)
                     _lhsOpp =
                         ppNestInfo ["Pattern","Underscore"] [ppShow pos_] [] []
                     -- self rule
@@ -477,10 +502,10 @@ sem_Patterns_Cons (T_Pattern hd_) (T_Patterns tl_) =
                      _tlIcopy :: Patterns
                      _tlIpp :: PP_Doc
                      _tlIppL :: ([PP_Doc])
-                     -- "AbstractSyntaxDump.ag"(line 54, column 33)
+                     -- "AbstractSyntaxDump.ag"(line 55, column 33)
                      _lhsOppL =
                          _hdIpp : _tlIppL
-                     -- use rule "AbstractSyntaxDump.ag"(line 16, column 58)
+                     -- use rule "AbstractSyntaxDump.ag"(line 17, column 58)
                      _lhsOpp =
                          _hdIpp >-< _tlIpp
                      -- self rule
@@ -499,10 +524,10 @@ sem_Patterns_Nil  =
     (T_Patterns (let _lhsOppL :: ([PP_Doc])
                      _lhsOpp :: PP_Doc
                      _lhsOcopy :: Patterns
-                     -- "AbstractSyntaxDump.ag"(line 55, column 33)
+                     -- "AbstractSyntaxDump.ag"(line 56, column 33)
                      _lhsOppL =
                          []
-                     -- use rule "AbstractSyntaxDump.ag"(line 16, column 58)
+                     -- use rule "AbstractSyntaxDump.ag"(line 17, column 58)
                      _lhsOpp =
                          empty
                      -- self rule
@@ -519,16 +544,16 @@ sem_Patterns_Nil  =
          pp                   : PP_Doc
    alternatives:
       alternative Production:
-         child nt             : {Nonterminal}
-         child inh            : {Attributes}
-         child syn            : {Attributes}
-         child alts           : Alternatives
+         child con            : {ConstructorIdent}
+         child children       : Children
+         child rules          : Rules
+         child typeSigs       : TypeSigs
 -}
 -- cata
 sem_Production :: Production ->
                   T_Production
-sem_Production (Production _nt _inh _syn _alts) =
-    (sem_Production_Production _nt _inh _syn (sem_Alternatives _alts))
+sem_Production (Production _con _children _rules _typeSigs) =
+    (sem_Production_Production _con (sem_Children _children) (sem_Rules _rules) (sem_TypeSigs _typeSigs))
 -- semantic domain
 newtype T_Production = T_Production (( PP_Doc))
 data Inh_Production = Inh_Production {}
@@ -537,20 +562,28 @@ wrap_Production (T_Production sem) (Inh_Production ) =
     (let ( _lhsOpp) =
              (sem )
      in  (Syn_Production _lhsOpp))
-sem_Production_Production :: Nonterminal ->
-                             Attributes ->
-                             Attributes ->
-                             T_Alternatives ->
+sem_Production_Production :: ConstructorIdent ->
+                             T_Children ->
+                             T_Rules ->
+                             T_TypeSigs ->
                              T_Production
-sem_Production_Production nt_ inh_ syn_ (T_Alternatives alts_) =
+sem_Production_Production con_ (T_Children children_) (T_Rules rules_) (T_TypeSigs typeSigs_) =
     (T_Production (let _lhsOpp :: PP_Doc
-                       _altsIpp :: PP_Doc
-                       _altsIppL :: ([PP_Doc])
-                       -- "AbstractSyntaxDump.ag"(line 28, column 25)
+                       _childrenIpp :: PP_Doc
+                       _childrenIppL :: ([PP_Doc])
+                       _rulesIpp :: PP_Doc
+                       _rulesIppL :: ([PP_Doc])
+                       _typeSigsIpp :: PP_Doc
+                       _typeSigsIppL :: ([PP_Doc])
+                       -- "AbstractSyntaxDump.ag"(line 32, column 25)
                        _lhsOpp =
-                           ppNestInfo ["Production","Production"] [pp nt_] [ppF "inh" $ ppMap inh_, ppF "syn" $ ppMap syn_, ppF "alts" $ ppVList _altsIppL] []
-                       ( _altsIpp,_altsIppL) =
-                           (alts_ )
+                           ppNestInfo ["Production","Production"] [pp con_] [ppF "children" $ ppVList _childrenIppL,ppF "rules" $ ppVList _rulesIppL,ppF "typeSigs" $ ppVList _typeSigsIppL] []
+                       ( _childrenIpp,_childrenIppL) =
+                           (children_ )
+                       ( _rulesIpp,_rulesIppL) =
+                           (rules_ )
+                       ( _typeSigsIpp,_typeSigsIppL) =
+                           (typeSigs_ )
                    in  ( _lhsOpp)))
 -- Productions -------------------------------------------------
 {-
@@ -586,10 +619,10 @@ sem_Productions_Cons (T_Production hd_) (T_Productions tl_) =
                         _hdIpp :: PP_Doc
                         _tlIpp :: PP_Doc
                         _tlIppL :: ([PP_Doc])
-                        -- "AbstractSyntaxDump.ag"(line 74, column 33)
+                        -- "AbstractSyntaxDump.ag"(line 71, column 33)
                         _lhsOppL =
                             _hdIpp : _tlIppL
-                        -- use rule "AbstractSyntaxDump.ag"(line 16, column 58)
+                        -- use rule "AbstractSyntaxDump.ag"(line 17, column 58)
                         _lhsOpp =
                             _hdIpp >-< _tlIpp
                         ( _hdIpp) =
@@ -601,10 +634,10 @@ sem_Productions_Nil :: T_Productions
 sem_Productions_Nil  =
     (T_Productions (let _lhsOppL :: ([PP_Doc])
                         _lhsOpp :: PP_Doc
-                        -- "AbstractSyntaxDump.ag"(line 75, column 33)
+                        -- "AbstractSyntaxDump.ag"(line 72, column 33)
                         _lhsOppL =
                             []
-                        -- use rule "AbstractSyntaxDump.ag"(line 16, column 58)
+                        -- use rule "AbstractSyntaxDump.ag"(line 17, column 58)
                         _lhsOpp =
                             empty
                     in  ( _lhsOpp,_lhsOppL)))
@@ -643,7 +676,7 @@ sem_Rule_Rule (T_Pattern pattern_) (T_Expression rhs_) owrt_ origin_ =
                  _patternIcopy :: Pattern
                  _patternIpp :: PP_Doc
                  _rhsIpp :: PP_Doc
-                 -- "AbstractSyntaxDump.ag"(line 37, column 33)
+                 -- "AbstractSyntaxDump.ag"(line 38, column 33)
                  _lhsOpp =
                      ppNestInfo ["Rule","Rule"] [ppShow owrt_, pp origin_] [ppF "pattern" $ _patternIpp, ppF "rhs" $ _rhsIpp] []
                  ( _patternIcopy,_patternIpp) =
@@ -685,10 +718,10 @@ sem_Rules_Cons (T_Rule hd_) (T_Rules tl_) =
                   _hdIpp :: PP_Doc
                   _tlIpp :: PP_Doc
                   _tlIppL :: ([PP_Doc])
-                  -- "AbstractSyntaxDump.ag"(line 62, column 33)
+                  -- "AbstractSyntaxDump.ag"(line 63, column 33)
                   _lhsOppL =
                       _hdIpp : _tlIppL
-                  -- use rule "AbstractSyntaxDump.ag"(line 16, column 58)
+                  -- use rule "AbstractSyntaxDump.ag"(line 17, column 58)
                   _lhsOpp =
                       _hdIpp >-< _tlIpp
                   ( _hdIpp) =
@@ -700,10 +733,10 @@ sem_Rules_Nil :: T_Rules
 sem_Rules_Nil  =
     (T_Rules (let _lhsOppL :: ([PP_Doc])
                   _lhsOpp :: PP_Doc
-                  -- "AbstractSyntaxDump.ag"(line 63, column 33)
+                  -- "AbstractSyntaxDump.ag"(line 64, column 33)
                   _lhsOppL =
                       []
-                  -- use rule "AbstractSyntaxDump.ag"(line 16, column 58)
+                  -- use rule "AbstractSyntaxDump.ag"(line 17, column 58)
                   _lhsOpp =
                       empty
               in  ( _lhsOpp,_lhsOppL)))
@@ -714,7 +747,7 @@ sem_Rules_Nil  =
          pp                   : PP_Doc
    alternatives:
       alternative TypeSig:
-         child name           : {Name}
+         child name           : {Identifier}
          child tp             : {Type}
 -}
 -- cata
@@ -730,12 +763,12 @@ wrap_TypeSig (T_TypeSig sem) (Inh_TypeSig ) =
     (let ( _lhsOpp) =
              (sem )
      in  (Syn_TypeSig _lhsOpp))
-sem_TypeSig_TypeSig :: Name ->
+sem_TypeSig_TypeSig :: Identifier ->
                        Type ->
                        T_TypeSig
 sem_TypeSig_TypeSig name_ tp_ =
     (T_TypeSig (let _lhsOpp :: PP_Doc
-                    -- "AbstractSyntaxDump.ag"(line 40, column 33)
+                    -- "AbstractSyntaxDump.ag"(line 41, column 33)
                     _lhsOpp =
                         ppNestInfo ["TypeSig","TypeSig"] [pp name_, ppShow tp_] [] []
                 in  ( _lhsOpp)))
@@ -773,10 +806,10 @@ sem_TypeSigs_Cons (T_TypeSig hd_) (T_TypeSigs tl_) =
                      _hdIpp :: PP_Doc
                      _tlIpp :: PP_Doc
                      _tlIppL :: ([PP_Doc])
-                     -- "AbstractSyntaxDump.ag"(line 58, column 33)
+                     -- "AbstractSyntaxDump.ag"(line 59, column 33)
                      _lhsOppL =
                          _hdIpp : _tlIppL
-                     -- use rule "AbstractSyntaxDump.ag"(line 16, column 58)
+                     -- use rule "AbstractSyntaxDump.ag"(line 17, column 58)
                      _lhsOpp =
                          _hdIpp >-< _tlIpp
                      ( _hdIpp) =
@@ -788,10 +821,10 @@ sem_TypeSigs_Nil :: T_TypeSigs
 sem_TypeSigs_Nil  =
     (T_TypeSigs (let _lhsOppL :: ([PP_Doc])
                      _lhsOpp :: PP_Doc
-                     -- "AbstractSyntaxDump.ag"(line 59, column 33)
+                     -- "AbstractSyntaxDump.ag"(line 60, column 33)
                      _lhsOppL =
                          []
-                     -- use rule "AbstractSyntaxDump.ag"(line 16, column 58)
+                     -- use rule "AbstractSyntaxDump.ag"(line 17, column 58)
                      _lhsOpp =
                          empty
                  in  ( _lhsOpp,_lhsOppL)))
