@@ -16,17 +16,17 @@ import Data.Set(Set)
 
 -- Patterns.ag imports
 import UU.Scanner.Position(Pos)
-import CommonTypes (Constructor,Name)
+import CommonTypes (ConstructorIdent,Identifier)
 
-ppChild :: (Name,Type,Bool) -> PP_Doc
+ppChild :: (Identifier,Type,Bool) -> PP_Doc
 ppChild (nm,tp,b)
   = pp nm >#< "::" >#< pp (show tp) >#< "<" >|< ppBool b >|< ">"
   
-ppVertexMap :: Map Int (Name,Name,Maybe Type) -> PP_Doc
+ppVertexMap :: Map Int (Identifier,Identifier,Maybe Type) -> PP_Doc
 ppVertexMap m
   = ppVList [ ppF (show k) $ ppAttr v | (k,v) <- Map.toList m ]
 
-ppAttr :: (Name,Name,Maybe Type) -> PP_Doc
+ppAttr :: (Identifier,Identifier,Maybe Type) -> PP_Doc
 ppAttr (fld,nm,mTp)
   = pp fld >|< "." >|< pp nm >#<
     case mTp of
@@ -43,102 +43,6 @@ ppMaybeShow Nothing  = pp "_"
 
 ppStrings :: [String] -> PP_Doc
 ppStrings = vlist
--- CAlternative ------------------------------------------------
-{-
-   visit 0:
-      synthesized attribute:
-         pp                   : PP_Doc
-   alternatives:
-      alternative CAlternative:
-         child con            : {Constructor}
-         child visits         : CVisits
-         child children       : {[(Name,Type,Bool)]}
-         child terminals      : {[Name]}
--}
--- cata
-sem_CAlternative :: CAlternative ->
-                    T_CAlternative
-sem_CAlternative (CAlternative _con _visits _children _terminals) =
-    (sem_CAlternative_CAlternative _con (sem_CVisits _visits) _children _terminals)
--- semantic domain
-newtype T_CAlternative = T_CAlternative (( PP_Doc))
-data Inh_CAlternative = Inh_CAlternative {}
-data Syn_CAlternative = Syn_CAlternative {pp_Syn_CAlternative :: PP_Doc}
-wrap_CAlternative (T_CAlternative sem) (Inh_CAlternative ) =
-    (let ( _lhsOpp) =
-             (sem )
-     in  (Syn_CAlternative _lhsOpp))
-sem_CAlternative_CAlternative :: Constructor ->
-                                 T_CVisits ->
-                                 ([(Name,Type,Bool)]) ->
-                                 ([Name]) ->
-                                 T_CAlternative
-sem_CAlternative_CAlternative con_ (T_CVisits visits_) children_ terminals_ =
-    (T_CAlternative (let _lhsOpp :: PP_Doc
-                         _visitsIpp :: PP_Doc
-                         _visitsIppL :: ([PP_Doc])
-                         -- "CodeSyntaxDump.ag"(line 63, column 25)
-                         _lhsOpp =
-                             ppNestInfo ["CAlternative","CAlternative"] [pp con_] [ppF "visits" $ ppVList _visitsIppL, ppF "children" $ ppVList (map ppChild children_),ppF "terminals" $ ppVList (map ppShow terminals_)] []
-                         ( _visitsIpp,_visitsIppL) =
-                             (visits_ )
-                     in  ( _lhsOpp)))
--- CAlternatives -----------------------------------------------
-{-
-   visit 0:
-      synthesized attributes:
-         pp                   : PP_Doc
-         ppL                  : [PP_Doc]
-   alternatives:
-      alternative Cons:
-         child hd             : CAlternative
-         child tl             : CAlternatives
-      alternative Nil:
--}
--- cata
-sem_CAlternatives :: CAlternatives ->
-                     T_CAlternatives
-sem_CAlternatives list =
-    (Prelude.foldr sem_CAlternatives_Cons sem_CAlternatives_Nil (Prelude.map sem_CAlternative list))
--- semantic domain
-newtype T_CAlternatives = T_CAlternatives (( PP_Doc,([PP_Doc])))
-data Inh_CAlternatives = Inh_CAlternatives {}
-data Syn_CAlternatives = Syn_CAlternatives {pp_Syn_CAlternatives :: PP_Doc,ppL_Syn_CAlternatives :: [PP_Doc]}
-wrap_CAlternatives (T_CAlternatives sem) (Inh_CAlternatives ) =
-    (let ( _lhsOpp,_lhsOppL) =
-             (sem )
-     in  (Syn_CAlternatives _lhsOpp _lhsOppL))
-sem_CAlternatives_Cons :: T_CAlternative ->
-                          T_CAlternatives ->
-                          T_CAlternatives
-sem_CAlternatives_Cons (T_CAlternative hd_) (T_CAlternatives tl_) =
-    (T_CAlternatives (let _lhsOppL :: ([PP_Doc])
-                          _lhsOpp :: PP_Doc
-                          _hdIpp :: PP_Doc
-                          _tlIpp :: PP_Doc
-                          _tlIppL :: ([PP_Doc])
-                          -- "CodeSyntaxDump.ag"(line 94, column 33)
-                          _lhsOppL =
-                              _hdIpp : _tlIppL
-                          -- use rule "CodeSyntaxDump.ag"(line 44, column 40)
-                          _lhsOpp =
-                              _hdIpp >-< _tlIpp
-                          ( _hdIpp) =
-                              (hd_ )
-                          ( _tlIpp,_tlIppL) =
-                              (tl_ )
-                      in  ( _lhsOpp,_lhsOppL)))
-sem_CAlternatives_Nil :: T_CAlternatives
-sem_CAlternatives_Nil  =
-    (T_CAlternatives (let _lhsOppL :: ([PP_Doc])
-                          _lhsOpp :: PP_Doc
-                          -- "CodeSyntaxDump.ag"(line 95, column 33)
-                          _lhsOppL =
-                              []
-                          -- use rule "CodeSyntaxDump.ag"(line 44, column 40)
-                          _lhsOpp =
-                              empty
-                      in  ( _lhsOpp,_lhsOppL)))
 -- CGrammar ----------------------------------------------------
 {-
    visit 0:
@@ -148,15 +52,17 @@ sem_CAlternatives_Nil  =
       alternative CGrammar:
          child typeSyns       : {TypeSyns}
          child derivings      : {Derivings}
-         child wrappers       : {Set Nonterminal}
-         child prods          : CProductions
+         child wrappers       : {Set NontermIdent}
+         child nonts          : CNonterminals
          child pragmas        : {PragmaMap}
+         child paramMap       : {ParamMap}
+         child contextMap     : {ContextMap}
 -}
 -- cata
 sem_CGrammar :: CGrammar ->
                 T_CGrammar
-sem_CGrammar (CGrammar _typeSyns _derivings _wrappers _prods _pragmas) =
-    (sem_CGrammar_CGrammar _typeSyns _derivings _wrappers (sem_CProductions _prods) _pragmas)
+sem_CGrammar (CGrammar _typeSyns _derivings _wrappers _nonts _pragmas _paramMap _contextMap) =
+    (sem_CGrammar_CGrammar _typeSyns _derivings _wrappers (sem_CNonterminals _nonts) _pragmas _paramMap _contextMap)
 -- semantic domain
 newtype T_CGrammar = T_CGrammar (( PP_Doc))
 data Inh_CGrammar = Inh_CGrammar {}
@@ -167,23 +73,25 @@ wrap_CGrammar (T_CGrammar sem) (Inh_CGrammar ) =
      in  (Syn_CGrammar _lhsOpp))
 sem_CGrammar_CGrammar :: TypeSyns ->
                          Derivings ->
-                         (Set Nonterminal) ->
-                         T_CProductions ->
+                         (Set NontermIdent) ->
+                         T_CNonterminals ->
                          PragmaMap ->
+                         ParamMap ->
+                         ContextMap ->
                          T_CGrammar
-sem_CGrammar_CGrammar typeSyns_ derivings_ wrappers_ (T_CProductions prods_) pragmas_ =
+sem_CGrammar_CGrammar typeSyns_ derivings_ wrappers_ (T_CNonterminals nonts_) pragmas_ paramMap_ contextMap_ =
     (T_CGrammar (let _lhsOpp :: PP_Doc
-                     _prodsIpp :: PP_Doc
-                     _prodsIppL :: ([PP_Doc])
+                     _nontsIpp :: PP_Doc
+                     _nontsIppL :: ([PP_Doc])
                      -- "CodeSyntaxDump.ag"(line 47, column 21)
                      _lhsOpp =
                          ppNestInfo ["CGrammar","CGrammar"] []
                                     [ ppF "typeSyns"  $ ppAssocL typeSyns_
                                     , ppF "derivings" $ ppMap $ derivings_
-                                    , ppF "prods"     $ ppVList _prodsIppL
+                                    , ppF "nonts"     $ ppVList _nontsIppL
                                     ] []
-                     ( _prodsIpp,_prodsIppL) =
-                         (prods_ )
+                     ( _nontsIpp,_nontsIppL) =
+                         (nonts_ )
                  in  ( _lhsOpp)))
 -- CInterface --------------------------------------------------
 {-
@@ -219,6 +127,109 @@ sem_CInterface_CInterface (T_CSegments seg_) =
                        ( _segIpp,_segIppL) =
                            (seg_ )
                    in  ( _lhsOpp)))
+-- CNonterminal ------------------------------------------------
+{-
+   visit 0:
+      synthesized attribute:
+         pp                   : PP_Doc
+   alternatives:
+      alternative CNonterminal:
+         child nt             : {NontermIdent}
+         child params         : {[Identifier]}
+         child inh            : {Attributes}
+         child syn            : {Attributes}
+         child prods          : CProductions
+         child inter          : CInterface
+-}
+-- cata
+sem_CNonterminal :: CNonterminal ->
+                    T_CNonterminal
+sem_CNonterminal (CNonterminal _nt _params _inh _syn _prods _inter) =
+    (sem_CNonterminal_CNonterminal _nt _params _inh _syn (sem_CProductions _prods) (sem_CInterface _inter))
+-- semantic domain
+newtype T_CNonterminal = T_CNonterminal (( PP_Doc))
+data Inh_CNonterminal = Inh_CNonterminal {}
+data Syn_CNonterminal = Syn_CNonterminal {pp_Syn_CNonterminal :: PP_Doc}
+wrap_CNonterminal (T_CNonterminal sem) (Inh_CNonterminal ) =
+    (let ( _lhsOpp) =
+             (sem )
+     in  (Syn_CNonterminal _lhsOpp))
+sem_CNonterminal_CNonterminal :: NontermIdent ->
+                                 ([Identifier]) ->
+                                 Attributes ->
+                                 Attributes ->
+                                 T_CProductions ->
+                                 T_CInterface ->
+                                 T_CNonterminal
+sem_CNonterminal_CNonterminal nt_ params_ inh_ syn_ (T_CProductions prods_) (T_CInterface inter_) =
+    (T_CNonterminal (let _lhsOpp :: PP_Doc
+                         _prodsIpp :: PP_Doc
+                         _prodsIppL :: ([PP_Doc])
+                         _interIpp :: PP_Doc
+                         -- "CodeSyntaxDump.ag"(line 54, column 33)
+                         _lhsOpp =
+                             ppNestInfo ["CNonterminal","CNonterminal"] (pp nt_ : map pp params_) [ppF "inh" $ ppMap inh_, ppF "syn" $ ppMap syn_, ppF "prods" $ ppVList _prodsIppL, ppF "inter" _interIpp] []
+                         ( _prodsIpp,_prodsIppL) =
+                             (prods_ )
+                         ( _interIpp) =
+                             (inter_ )
+                     in  ( _lhsOpp)))
+-- CNonterminals -----------------------------------------------
+{-
+   visit 0:
+      synthesized attributes:
+         pp                   : PP_Doc
+         ppL                  : [PP_Doc]
+   alternatives:
+      alternative Cons:
+         child hd             : CNonterminal
+         child tl             : CNonterminals
+      alternative Nil:
+-}
+-- cata
+sem_CNonterminals :: CNonterminals ->
+                     T_CNonterminals
+sem_CNonterminals list =
+    (Prelude.foldr sem_CNonterminals_Cons sem_CNonterminals_Nil (Prelude.map sem_CNonterminal list))
+-- semantic domain
+newtype T_CNonterminals = T_CNonterminals (( PP_Doc,([PP_Doc])))
+data Inh_CNonterminals = Inh_CNonterminals {}
+data Syn_CNonterminals = Syn_CNonterminals {pp_Syn_CNonterminals :: PP_Doc,ppL_Syn_CNonterminals :: [PP_Doc]}
+wrap_CNonterminals (T_CNonterminals sem) (Inh_CNonterminals ) =
+    (let ( _lhsOpp,_lhsOppL) =
+             (sem )
+     in  (Syn_CNonterminals _lhsOpp _lhsOppL))
+sem_CNonterminals_Cons :: T_CNonterminal ->
+                          T_CNonterminals ->
+                          T_CNonterminals
+sem_CNonterminals_Cons (T_CNonterminal hd_) (T_CNonterminals tl_) =
+    (T_CNonterminals (let _lhsOppL :: ([PP_Doc])
+                          _lhsOpp :: PP_Doc
+                          _hdIpp :: PP_Doc
+                          _tlIpp :: PP_Doc
+                          _tlIppL :: ([PP_Doc])
+                          -- "CodeSyntaxDump.ag"(line 102, column 33)
+                          _lhsOppL =
+                              _hdIpp : _tlIppL
+                          -- use rule "CodeSyntaxDump.ag"(line 44, column 40)
+                          _lhsOpp =
+                              _hdIpp >-< _tlIpp
+                          ( _hdIpp) =
+                              (hd_ )
+                          ( _tlIpp,_tlIppL) =
+                              (tl_ )
+                      in  ( _lhsOpp,_lhsOppL)))
+sem_CNonterminals_Nil :: T_CNonterminals
+sem_CNonterminals_Nil  =
+    (T_CNonterminals (let _lhsOppL :: ([PP_Doc])
+                          _lhsOpp :: PP_Doc
+                          -- "CodeSyntaxDump.ag"(line 103, column 33)
+                          _lhsOppL =
+                              []
+                          -- use rule "CodeSyntaxDump.ag"(line 44, column 40)
+                          _lhsOpp =
+                              empty
+                      in  ( _lhsOpp,_lhsOppL)))
 -- CProduction -------------------------------------------------
 {-
    visit 0:
@@ -226,17 +237,16 @@ sem_CInterface_CInterface (T_CSegments seg_) =
          pp                   : PP_Doc
    alternatives:
       alternative CProduction:
-         child nt             : {Nonterminal}
-         child inh            : {Attributes}
-         child syn            : {Attributes}
-         child alts           : CAlternatives
-         child inter          : CInterface
+         child con            : {ConstructorIdent}
+         child visits         : CVisits
+         child children       : {[(Identifier,Type,Bool)]}
+         child terminals      : {[Identifier]}
 -}
 -- cata
 sem_CProduction :: CProduction ->
                    T_CProduction
-sem_CProduction (CProduction _nt _inh _syn _alts _inter) =
-    (sem_CProduction_CProduction _nt _inh _syn (sem_CAlternatives _alts) (sem_CInterface _inter))
+sem_CProduction (CProduction _con _visits _children _terminals) =
+    (sem_CProduction_CProduction _con (sem_CVisits _visits) _children _terminals)
 -- semantic domain
 newtype T_CProduction = T_CProduction (( PP_Doc))
 data Inh_CProduction = Inh_CProduction {}
@@ -245,24 +255,20 @@ wrap_CProduction (T_CProduction sem) (Inh_CProduction ) =
     (let ( _lhsOpp) =
              (sem )
      in  (Syn_CProduction _lhsOpp))
-sem_CProduction_CProduction :: Nonterminal ->
-                               Attributes ->
-                               Attributes ->
-                               T_CAlternatives ->
-                               T_CInterface ->
+sem_CProduction_CProduction :: ConstructorIdent ->
+                               T_CVisits ->
+                               ([(Identifier,Type,Bool)]) ->
+                               ([Identifier]) ->
                                T_CProduction
-sem_CProduction_CProduction nt_ inh_ syn_ (T_CAlternatives alts_) (T_CInterface inter_) =
+sem_CProduction_CProduction con_ (T_CVisits visits_) children_ terminals_ =
     (T_CProduction (let _lhsOpp :: PP_Doc
-                        _altsIpp :: PP_Doc
-                        _altsIppL :: ([PP_Doc])
-                        _interIpp :: PP_Doc
-                        -- "CodeSyntaxDump.ag"(line 54, column 25)
+                        _visitsIpp :: PP_Doc
+                        _visitsIppL :: ([PP_Doc])
+                        -- "CodeSyntaxDump.ag"(line 63, column 17)
                         _lhsOpp =
-                            ppNestInfo ["CProduction","CProduction"] [pp nt_] [ppF "inh" $ ppMap inh_, ppF "syn" $ ppMap syn_, ppF "alts" $ ppVList _altsIppL, ppF "inter" _interIpp] []
-                        ( _altsIpp,_altsIppL) =
-                            (alts_ )
-                        ( _interIpp) =
-                            (inter_ )
+                            ppNestInfo ["CProduction","CProduction"] [pp con_] [ppF "visits" $ ppVList _visitsIppL, ppF "children" $ ppVList (map ppChild children_),ppF "terminals" $ ppVList (map ppShow terminals_)] []
+                        ( _visitsIpp,_visitsIppL) =
+                            (visits_ )
                     in  ( _lhsOpp)))
 -- CProductions ------------------------------------------------
 {-
@@ -298,7 +304,7 @@ sem_CProductions_Cons (T_CProduction hd_) (T_CProductions tl_) =
                          _hdIpp :: PP_Doc
                          _tlIpp :: PP_Doc
                          _tlIppL :: ([PP_Doc])
-                         -- "CodeSyntaxDump.ag"(line 102, column 33)
+                         -- "CodeSyntaxDump.ag"(line 94, column 33)
                          _lhsOppL =
                              _hdIpp : _tlIppL
                          -- use rule "CodeSyntaxDump.ag"(line 44, column 40)
@@ -313,7 +319,7 @@ sem_CProductions_Nil :: T_CProductions
 sem_CProductions_Nil  =
     (T_CProductions (let _lhsOppL :: ([PP_Doc])
                          _lhsOpp :: PP_Doc
-                         -- "CodeSyntaxDump.ag"(line 103, column 33)
+                         -- "CodeSyntaxDump.ag"(line 95, column 33)
                          _lhsOppL =
                              []
                          -- use rule "CodeSyntaxDump.ag"(line 44, column 40)
@@ -327,27 +333,27 @@ sem_CProductions_Nil  =
          pp                   : PP_Doc
    alternatives:
       alternative CChildVisit:
-         child name           : {Name}
-         child nt             : {Nonterminal}
+         child name           : {Identifier}
+         child nt             : {NontermIdent}
          child nr             : {Int}
          child inh            : {Attributes}
          child syn            : {Attributes}
          child isLast         : {Bool}
       alternative CRule:
-         child name           : {Name}
+         child name           : {Identifier}
          child isIn           : {Bool}
          child hasCode        : {Bool}
-         child nt             : {Nonterminal}
-         child con            : {Constructor}
-         child field          : {Name}
-         child childnt        : {Maybe Nonterminal}
+         child nt             : {NontermIdent}
+         child con            : {ConstructorIdent}
+         child field          : {Identifier}
+         child childnt        : {Maybe NontermIdent}
          child tp             : {Maybe Type}
          child pattern        : Pattern
          child rhs            : {[String]}
-         child defines        : {Map Int (Name,Name,Maybe Type)}
+         child defines        : {Map Int (Identifier,Identifier,Maybe Type)}
          child owrt           : {Bool}
          child origin         : {String}
-         child uses           : {Set (Name, Name)}
+         child uses           : {Set (Identifier, Identifier)}
 -}
 -- cata
 sem_CRule :: CRule ->
@@ -364,8 +370,8 @@ wrap_CRule (T_CRule sem) (Inh_CRule ) =
     (let ( _lhsOpp) =
              (sem )
      in  (Syn_CRule _lhsOpp))
-sem_CRule_CChildVisit :: Name ->
-                         Nonterminal ->
+sem_CRule_CChildVisit :: Identifier ->
+                         NontermIdent ->
                          Int ->
                          Attributes ->
                          Attributes ->
@@ -377,20 +383,20 @@ sem_CRule_CChildVisit name_ nt_ nr_ inh_ syn_ isLast_ =
                   _lhsOpp =
                       ppNestInfo ["CRule","CChildVisit"] [pp name_] [ppF "nt" $ pp nt_, ppF "nr" $ ppShow nr_, ppF "inh" $ ppMap inh_, ppF "syn" $ ppMap syn_, ppF "last" $ ppBool isLast_] []
               in  ( _lhsOpp)))
-sem_CRule_CRule :: Name ->
+sem_CRule_CRule :: Identifier ->
                    Bool ->
                    Bool ->
-                   Nonterminal ->
-                   Constructor ->
-                   Name ->
-                   (Maybe Nonterminal) ->
+                   NontermIdent ->
+                   ConstructorIdent ->
+                   Identifier ->
+                   (Maybe NontermIdent) ->
                    (Maybe Type) ->
                    T_Pattern ->
                    ([String]) ->
-                   (Map Int (Name,Name,Maybe Type)) ->
+                   (Map Int (Identifier,Identifier,Maybe Type)) ->
                    Bool ->
                    String ->
-                   (Set (Name, Name)) ->
+                   (Set (Identifier, Identifier)) ->
                    T_CRule
 sem_CRule_CRule name_ isIn_ hasCode_ nt_ con_ field_ childnt_ tp_ (T_Pattern pattern_) rhs_ defines_ owrt_ origin_ uses_ =
     (T_CRule (let _lhsOpp :: PP_Doc
@@ -598,17 +604,21 @@ sem_CVisits_Nil  =
          pp                   : PP_Doc
    alternatives:
       alternative Alias:
-         child field          : {Name}
-         child attr           : {Name}
+         child field          : {Identifier}
+         child attr           : {Identifier}
          child pat            : Pattern
          child parts          : Patterns
          visit 0:
             local copy        : _
       alternative Constr:
-         child name           : {Constructor}
+         child name           : {ConstructorIdent}
          child pats           : Patterns
          visit 0:
             local copy        : _
+      alternative Irrefutable:
+         child pat            : Pattern
+         visit 0:
+            local copy        : _
       alternative Product:
          child pos            : {Pos}
          child pats           : Patterns
@@ -626,6 +636,8 @@ sem_Pattern (Alias _field _attr _pat _parts) =
     (sem_Pattern_Alias _field _attr (sem_Pattern _pat) (sem_Patterns _parts))
 sem_Pattern (Constr _name _pats) =
     (sem_Pattern_Constr _name (sem_Patterns _pats))
+sem_Pattern (Irrefutable _pat) =
+    (sem_Pattern_Irrefutable (sem_Pattern _pat))
 sem_Pattern (Product _pos _pats) =
     (sem_Pattern_Product _pos (sem_Patterns _pats))
 sem_Pattern (Underscore _pos) =
@@ -638,8 +650,8 @@ wrap_Pattern (T_Pattern sem) (Inh_Pattern ) =
     (let ( _lhsOcopy,_lhsOpp) =
              (sem )
      in  (Syn_Pattern _lhsOcopy _lhsOpp))
-sem_Pattern_Alias :: Name ->
-                     Name ->
+sem_Pattern_Alias :: Identifier ->
+                     Identifier ->
                      T_Pattern ->
                      T_Patterns ->
                      T_Pattern
@@ -665,7 +677,7 @@ sem_Pattern_Alias field_ attr_ (T_Pattern pat_) (T_Patterns parts_) =
                     ( _partsIcopy,_partsIpp,_partsIppL) =
                         (parts_ )
                 in  ( _lhsOcopy,_lhsOpp)))
-sem_Pattern_Constr :: Constructor ->
+sem_Pattern_Constr :: ConstructorIdent ->
                       T_Patterns ->
                       T_Pattern
 sem_Pattern_Constr name_ (T_Patterns pats_) =
@@ -686,6 +698,25 @@ sem_Pattern_Constr name_ (T_Patterns pats_) =
                     ( _patsIcopy,_patsIpp,_patsIppL) =
                         (pats_ )
                 in  ( _lhsOcopy,_lhsOpp)))
+sem_Pattern_Irrefutable :: T_Pattern ->
+                           T_Pattern
+sem_Pattern_Irrefutable (T_Pattern pat_) =
+    (T_Pattern (let _lhsOpp :: PP_Doc
+                    _lhsOcopy :: Pattern
+                    _patIcopy :: Pattern
+                    _patIpp :: PP_Doc
+                    -- use rule "CodeSyntaxDump.ag"(line 44, column 40)
+                    _lhsOpp =
+                        _patIpp
+                    -- self rule
+                    _copy =
+                        Irrefutable _patIcopy
+                    -- self rule
+                    _lhsOcopy =
+                        _copy
+                    ( _patIcopy,_patIpp) =
+                        (pat_ )
+                in  ( _lhsOcopy,_lhsOpp)))
 sem_Pattern_Product :: Pos ->
                        T_Patterns ->
                        T_Pattern
