@@ -1,12 +1,14 @@
+
 -- UUAGC 0.9.5 (Expression.ag)
 module Expression where
 
 import UU.Scanner.Position(Pos)
+import HsToken
 -- Expression --------------------------------------------------
 {-
    alternatives:
       alternative Expression:
          child pos            : {Pos}
-         child txt            : {String}
+         child tks            : {[HsToken]}
 -}
-data Expression = Expression (Pos) (String)
+data Expression = Expression (Pos) ([HsToken]) 
\ No newline at end of file
