@@ -1,3 +1,4 @@
+
 -- UUAGC 0.9.5 (Interfaces.ag)
 module Interfaces where
 
@@ -7,24 +8,24 @@ import SequentialTypes
 {-
    alternatives:
       alternative IRoot:
-         child inters         : Interfaces
+         child inters         : Interfaces 
 -}
-data IRoot = IRoot (Interfaces)
+data IRoot = IRoot (Interfaces) 
 -- Interface ---------------------------------------------------
 {-
    alternatives:
       alternative Interface:
-         child nt             : {Nonterminal}
-         child cons           : {[Constructor]}
-         child seg            : Segments
+         child nt             : {NontermIdent}
+         child cons           : {[ConstructorIdent]}
+         child seg            : Segments 
 -}
-data Interface = Interface (Nonterminal) ([Constructor]) (Segments)
+data Interface = Interface (NontermIdent) ([ConstructorIdent]) (Segments) 
 -- Interfaces --------------------------------------------------
 {-
    alternatives:
       alternative Cons:
-         child hd             : Interface
-         child tl             : Interfaces
+         child hd             : Interface 
+         child tl             : Interfaces 
       alternative Nil:
 -}
 type Interfaces = [Interface]
@@ -35,13 +36,13 @@ type Interfaces = [Interface]
          child inh            : {[Vertex]}
          child syn            : {[Vertex]}
 -}
-data Segment = Segment ([Vertex]) ([Vertex])
+data Segment = Segment ([Vertex]) ([Vertex]) 
 -- Segments ----------------------------------------------------
 {-
    alternatives:
       alternative Cons:
-         child hd             : Segment
-         child tl             : Segments
+         child hd             : Segment 
+         child tl             : Segments 
       alternative Nil:
 -}
-type Segments = [Segment]
+type Segments = [Segment]
\ No newline at end of file
