@@ -11,14 +11,14 @@ import ErrorMessages
 import CommonTypes
 import UU.Scanner.Position(Pos)
 
-isNTname allnts (Just (NT nt)) = nt `elem` allnts
-isNTname allnts _              = False
+isNTname allnts (Just (NT nt _)) = nt `elem` allnts
+isNTname allnts _                = False
 -- HsToken -----------------------------------------------------
 {-
    visit 0:
       inherited attributes:
-         allfields            : [(Name,Type,Bool)]
-         allnts               : [Name]
+         allfields            : [(Identifier,Type,Bool)]
+         allnts               : [Identifier]
          attrs                : [(Identifier,Identifier)]
          con                  : Identifier
          fieldnames           : [Identifier]
@@ -31,12 +31,16 @@ isNTname allnts _              = False
          usedLocals           : [Identifier]
    alternatives:
       alternative AGField:
-         child field          : {Name}
-         child attr           : {Name}
+         child field          : {Identifier}
+         child attr           : {Identifier}
          child pos            : {Pos}
+         child rdesc          : {Maybe String}
+         visit 0:
+            local addTrace    : _
       alternative AGLocal:
-         child var            : {Name}
+         child var            : {Identifier}
          child pos            : {Pos}
+         child rdesc          : {Maybe String}
          visit 0:
             local _tup1       : _
             local errors      : _
@@ -58,10 +62,10 @@ isNTname allnts _              = False
 -- cata
 sem_HsToken :: HsToken ->
                T_HsToken
-sem_HsToken (AGField _field _attr _pos) =
-    (sem_HsToken_AGField _field _attr _pos)
-sem_HsToken (AGLocal _var _pos) =
-    (sem_HsToken_AGLocal _var _pos)
+sem_HsToken (AGField _field _attr _pos _rdesc) =
+    (sem_HsToken_AGField _field _attr _pos _rdesc)
+sem_HsToken (AGLocal _var _pos _rdesc) =
+    (sem_HsToken_AGLocal _var _pos _rdesc)
 sem_HsToken (CharToken _value _pos) =
     (sem_HsToken_CharToken _value _pos)
 sem_HsToken (Err _mesg _pos) =
@@ -71,24 +75,25 @@ sem_HsToken (HsToken _value _pos) =
 sem_HsToken (StrToken _value _pos) =
     (sem_HsToken_StrToken _value _pos)
 -- semantic domain
-newtype T_HsToken = T_HsToken (([(Name,Type,Bool)]) ->
-                               ([Name]) ->
+newtype T_HsToken = T_HsToken (([(Identifier,Type,Bool)]) ->
+                               ([Identifier]) ->
                                ([(Identifier,Identifier)]) ->
                                Identifier ->
                                ([Identifier]) ->
                                Identifier ->
                                ( (Seq Error),((Pos,String)),([(Identifier,Identifier)]),(Seq Identifier),([Identifier])))
-data Inh_HsToken = Inh_HsToken {allfields_Inh_HsToken :: [(Name,Type,Bool)],allnts_Inh_HsToken :: [Name],attrs_Inh_HsToken :: [(Identifier,Identifier)],con_Inh_HsToken :: Identifier,fieldnames_Inh_HsToken :: [Identifier],nt_Inh_HsToken :: Identifier}
+data Inh_HsToken = Inh_HsToken {allfields_Inh_HsToken :: [(Identifier,Type,Bool)],allnts_Inh_HsToken :: [Identifier],attrs_Inh_HsToken :: [(Identifier,Identifier)],con_Inh_HsToken :: Identifier,fieldnames_Inh_HsToken :: [Identifier],nt_Inh_HsToken :: Identifier}
 data Syn_HsToken = Syn_HsToken {errors_Syn_HsToken :: Seq Error,tok_Syn_HsToken :: (Pos,String),usedAttrs_Syn_HsToken :: [(Identifier,Identifier)],usedFields_Syn_HsToken :: Seq Identifier,usedLocals_Syn_HsToken :: [Identifier]}
 wrap_HsToken (T_HsToken sem) (Inh_HsToken _lhsIallfields _lhsIallnts _lhsIattrs _lhsIcon _lhsIfieldnames _lhsInt) =
     (let ( _lhsOerrors,_lhsOtok,_lhsOusedAttrs,_lhsOusedFields,_lhsOusedLocals) =
              (sem _lhsIallfields _lhsIallnts _lhsIattrs _lhsIcon _lhsIfieldnames _lhsInt)
      in  (Syn_HsToken _lhsOerrors _lhsOtok _lhsOusedAttrs _lhsOusedFields _lhsOusedLocals))
-sem_HsToken_AGField :: Name ->
-                       Name ->
+sem_HsToken_AGField :: Identifier ->
+                       Identifier ->
                        Pos ->
+                       (Maybe String) ->
                        T_HsToken
-sem_HsToken_AGField field_ attr_ pos_ =
+sem_HsToken_AGField field_ attr_ pos_ rdesc_ =
     (T_HsToken (\ _lhsIallfields
                   _lhsIallnts
                   _lhsIattrs
@@ -100,30 +105,36 @@ sem_HsToken_AGField field_ attr_ pos_ =
                          _lhsOtok :: ((Pos,String))
                          _lhsOusedFields :: (Seq Identifier)
                          _lhsOusedLocals :: ([Identifier])
-                         -- "SemHsTokens.ag"(line 73, column 15)
+                         -- "SemHsTokens.ag"(line 72, column 15)
                          _lhsOerrors =
                              if (field_,attr_) `elem` _lhsIattrs
                                    then Seq.empty
                                    else if not(field_ `elem` (_LHS : _LOC: _lhsIfieldnames))
                                            then Seq.single (UndefChild _lhsInt _lhsIcon field_)
-                                           else Seq.single (UndefAttr _lhsInt _lhsIcon field_ attr_)
-                         -- "SemHsTokens.ag"(line 87, column 13)
+                                           else Seq.single (UndefAttr _lhsInt _lhsIcon field_ attr_ False)
+                         -- "SemHsTokens.ag"(line 86, column 13)
                          _lhsOusedAttrs =
                              [(field_,attr_)]
-                         -- "SemHsTokens.ag"(line 113, column 14)
+                         -- "SemHsTokens.ag"(line 113, column 8)
+                         _addTrace =
+                             case rdesc_ of
+                               Just d  -> \x -> "(trace " ++ show (d ++ " -> " ++ show field_ ++ "." ++ show attr_) ++ " (" ++ x ++ "))"
+                               Nothing -> id
+                         -- "SemHsTokens.ag"(line 116, column 8)
                          _lhsOtok =
-                             (pos_, attrname True field_ attr_)
-                         -- use rule "SemHsTokens.ag"(line 92, column 40)
+                             (pos_, _addTrace     $ attrname True field_ attr_)
+                         -- use rule "SemHsTokens.ag"(line 91, column 40)
                          _lhsOusedFields =
                              Seq.empty
-                         -- use rule "SemHsTokens.ag"(line 83, column 40)
+                         -- use rule "SemHsTokens.ag"(line 82, column 40)
                          _lhsOusedLocals =
                              []
                      in  ( _lhsOerrors,_lhsOtok,_lhsOusedAttrs,_lhsOusedFields,_lhsOusedLocals))))
-sem_HsToken_AGLocal :: Name ->
+sem_HsToken_AGLocal :: Identifier ->
                        Pos ->
+                       (Maybe String) ->
                        T_HsToken
-sem_HsToken_AGLocal var_ pos_ =
+sem_HsToken_AGLocal var_ pos_ rdesc_ =
     (T_HsToken (\ _lhsIallfields
                   _lhsIallnts
                   _lhsIattrs
@@ -135,7 +146,7 @@ sem_HsToken_AGLocal var_ pos_ =
                          _lhsOusedAttrs :: ([(Identifier,Identifier)])
                          _lhsOusedLocals :: ([Identifier])
                          _lhsOtok :: ((Pos,String))
-                         -- "SemHsTokens.ag"(line 63, column 19)
+                         -- "SemHsTokens.ag"(line 62, column 19)
                          __tup1 =
                              if var_ `elem` _lhsIfieldnames
                              then if  isNTname _lhsIallnts (lookup var_ (map (\(n,t,_) -> (n,t)) _lhsIallfields))
@@ -144,27 +155,27 @@ sem_HsToken_AGLocal var_ pos_ =
                              else if (_LOC,var_) `elem` _lhsIattrs
                              then (Seq.empty                                   , (pos_,locname   var_), [var_])
                              else (Seq.single(UndefLocal _lhsInt _lhsIcon var_), (pos_,locname   var_), []    )
-                         -- "SemHsTokens.ag"(line 63, column 19)
+                         -- "SemHsTokens.ag"(line 62, column 19)
                          (_errors,_,_) =
                              __tup1
-                         -- "SemHsTokens.ag"(line 63, column 19)
+                         -- "SemHsTokens.ag"(line 62, column 19)
                          (_,_tok,_) =
                              __tup1
-                         -- "SemHsTokens.ag"(line 63, column 19)
+                         -- "SemHsTokens.ag"(line 62, column 19)
                          (_,_,_usedLocals) =
                              __tup1
-                         -- "SemHsTokens.ag"(line 95, column 13)
+                         -- "SemHsTokens.ag"(line 94, column 13)
                          _lhsOusedFields =
                              if var_ `elem` _lhsIfieldnames
                               then Seq.single var_
                               else Seq.empty
-                         -- use rule "SemHsTokens.ag"(line 42, column 37)
+                         -- use rule "SemHsTokens.ag"(line 41, column 37)
                          _lhsOerrors =
                              _errors
-                         -- use rule "SemHsTokens.ag"(line 84, column 40)
+                         -- use rule "SemHsTokens.ag"(line 83, column 40)
                          _lhsOusedAttrs =
                              []
-                         -- use rule "SemHsTokens.ag"(line 83, column 40)
+                         -- use rule "SemHsTokens.ag"(line 82, column 40)
                          _lhsOusedLocals =
                              _usedLocals
                          -- copy rule (from local)
@@ -186,22 +197,22 @@ sem_HsToken_CharToken value_ pos_ =
                          _lhsOusedAttrs :: ([(Identifier,Identifier)])
                          _lhsOusedFields :: (Seq Identifier)
                          _lhsOusedLocals :: ([Identifier])
-                         -- "SemHsTokens.ag"(line 117, column 16)
+                         -- "SemHsTokens.ag"(line 120, column 16)
                          _lhsOtok =
                              (pos_, if null value_
                                        then ""
                                        else showCharShort (head value_)
                              )
-                         -- use rule "SemHsTokens.ag"(line 42, column 37)
+                         -- use rule "SemHsTokens.ag"(line 41, column 37)
                          _lhsOerrors =
                              Seq.empty
-                         -- use rule "SemHsTokens.ag"(line 84, column 40)
+                         -- use rule "SemHsTokens.ag"(line 83, column 40)
                          _lhsOusedAttrs =
                              []
-                         -- use rule "SemHsTokens.ag"(line 92, column 40)
+                         -- use rule "SemHsTokens.ag"(line 91, column 40)
                          _lhsOusedFields =
                              Seq.empty
-                         -- use rule "SemHsTokens.ag"(line 83, column 40)
+                         -- use rule "SemHsTokens.ag"(line 82, column 40)
                          _lhsOusedLocals =
                              []
                      in  ( _lhsOerrors,_lhsOtok,_lhsOusedAttrs,_lhsOusedFields,_lhsOusedLocals))))
@@ -220,20 +231,20 @@ sem_HsToken_Err mesg_ pos_ =
                          _lhsOusedAttrs :: ([(Identifier,Identifier)])
                          _lhsOusedFields :: (Seq Identifier)
                          _lhsOusedLocals :: ([Identifier])
-                         -- "SemHsTokens.ag"(line 49, column 9)
+                         -- "SemHsTokens.ag"(line 48, column 9)
                          _lhsOerrors =
                              let m = text mesg_
                              in Seq.single (CustomError False pos_ m)
-                         -- "SemHsTokens.ag"(line 123, column 16)
+                         -- "SemHsTokens.ag"(line 126, column 16)
                          _lhsOtok =
                              (pos_, "")
-                         -- use rule "SemHsTokens.ag"(line 84, column 40)
+                         -- use rule "SemHsTokens.ag"(line 83, column 40)
                          _lhsOusedAttrs =
                              []
-                         -- use rule "SemHsTokens.ag"(line 92, column 40)
+                         -- use rule "SemHsTokens.ag"(line 91, column 40)
                          _lhsOusedFields =
                              Seq.empty
-                         -- use rule "SemHsTokens.ag"(line 83, column 40)
+                         -- use rule "SemHsTokens.ag"(line 82, column 40)
                          _lhsOusedLocals =
                              []
                      in  ( _lhsOerrors,_lhsOtok,_lhsOusedAttrs,_lhsOusedFields,_lhsOusedLocals))))
@@ -252,19 +263,19 @@ sem_HsToken_HsToken value_ pos_ =
                          _lhsOusedAttrs :: ([(Identifier,Identifier)])
                          _lhsOusedFields :: (Seq Identifier)
                          _lhsOusedLocals :: ([Identifier])
-                         -- "SemHsTokens.ag"(line 115, column 14)
+                         -- "SemHsTokens.ag"(line 118, column 14)
                          _lhsOtok =
                              (pos_, value_)
-                         -- use rule "SemHsTokens.ag"(line 42, column 37)
+                         -- use rule "SemHsTokens.ag"(line 41, column 37)
                          _lhsOerrors =
                              Seq.empty
-                         -- use rule "SemHsTokens.ag"(line 84, column 40)
+                         -- use rule "SemHsTokens.ag"(line 83, column 40)
                          _lhsOusedAttrs =
                              []
-                         -- use rule "SemHsTokens.ag"(line 92, column 40)
+                         -- use rule "SemHsTokens.ag"(line 91, column 40)
                          _lhsOusedFields =
                              Seq.empty
-                         -- use rule "SemHsTokens.ag"(line 83, column 40)
+                         -- use rule "SemHsTokens.ag"(line 82, column 40)
                          _lhsOusedLocals =
                              []
                      in  ( _lhsOerrors,_lhsOtok,_lhsOusedAttrs,_lhsOusedFields,_lhsOusedLocals))))
@@ -283,19 +294,19 @@ sem_HsToken_StrToken value_ pos_ =
                          _lhsOusedAttrs :: ([(Identifier,Identifier)])
                          _lhsOusedFields :: (Seq Identifier)
                          _lhsOusedLocals :: ([Identifier])
-                         -- "SemHsTokens.ag"(line 122, column 16)
+                         -- "SemHsTokens.ag"(line 125, column 16)
                          _lhsOtok =
                              (pos_, showStrShort value_)
-                         -- use rule "SemHsTokens.ag"(line 42, column 37)
+                         -- use rule "SemHsTokens.ag"(line 41, column 37)
                          _lhsOerrors =
                              Seq.empty
-                         -- use rule "SemHsTokens.ag"(line 84, column 40)
+                         -- use rule "SemHsTokens.ag"(line 83, column 40)
                          _lhsOusedAttrs =
                              []
-                         -- use rule "SemHsTokens.ag"(line 92, column 40)
+                         -- use rule "SemHsTokens.ag"(line 91, column 40)
                          _lhsOusedFields =
                              Seq.empty
-                         -- use rule "SemHsTokens.ag"(line 83, column 40)
+                         -- use rule "SemHsTokens.ag"(line 82, column 40)
                          _lhsOusedLocals =
                              []
                      in  ( _lhsOerrors,_lhsOtok,_lhsOusedAttrs,_lhsOusedFields,_lhsOusedLocals))))
@@ -303,8 +314,8 @@ sem_HsToken_StrToken value_ pos_ =
 {-
    visit 0:
       inherited attributes:
-         allfields            : [(Name,Type,Bool)]
-         allnts               : [Name]
+         allfields            : [(Identifier,Type,Bool)]
+         allnts               : [Identifier]
          attrs                : [(Identifier,Identifier)]
          con                  : Identifier
          fieldnames           : [Identifier]
@@ -327,14 +338,14 @@ sem_HsTokens :: HsTokens ->
 sem_HsTokens list =
     (Prelude.foldr sem_HsTokens_Cons sem_HsTokens_Nil (Prelude.map sem_HsToken list))
 -- semantic domain
-newtype T_HsTokens = T_HsTokens (([(Name,Type,Bool)]) ->
-                                 ([Name]) ->
+newtype T_HsTokens = T_HsTokens (([(Identifier,Type,Bool)]) ->
+                                 ([Identifier]) ->
                                  ([(Identifier,Identifier)]) ->
                                  Identifier ->
                                  ([Identifier]) ->
                                  Identifier ->
                                  ( (Seq Error),([(Pos,String)]),([(Identifier,Identifier)]),(Seq Identifier),([Identifier])))
-data Inh_HsTokens = Inh_HsTokens {allfields_Inh_HsTokens :: [(Name,Type,Bool)],allnts_Inh_HsTokens :: [Name],attrs_Inh_HsTokens :: [(Identifier,Identifier)],con_Inh_HsTokens :: Identifier,fieldnames_Inh_HsTokens :: [Identifier],nt_Inh_HsTokens :: Identifier}
+data Inh_HsTokens = Inh_HsTokens {allfields_Inh_HsTokens :: [(Identifier,Type,Bool)],allnts_Inh_HsTokens :: [Identifier],attrs_Inh_HsTokens :: [(Identifier,Identifier)],con_Inh_HsTokens :: Identifier,fieldnames_Inh_HsTokens :: [Identifier],nt_Inh_HsTokens :: Identifier}
 data Syn_HsTokens = Syn_HsTokens {errors_Syn_HsTokens :: Seq Error,tks_Syn_HsTokens :: [(Pos,String)],usedAttrs_Syn_HsTokens :: [(Identifier,Identifier)],usedFields_Syn_HsTokens :: Seq Identifier,usedLocals_Syn_HsTokens :: [Identifier]}
 wrap_HsTokens (T_HsTokens sem) (Inh_HsTokens _lhsIallfields _lhsIallnts _lhsIattrs _lhsIcon _lhsIfieldnames _lhsInt) =
     (let ( _lhsOerrors,_lhsOtks,_lhsOusedAttrs,_lhsOusedFields,_lhsOusedLocals) =
@@ -355,14 +366,14 @@ sem_HsTokens_Cons (T_HsToken hd_) (T_HsTokens tl_) =
                           _lhsOusedAttrs :: ([(Identifier,Identifier)])
                           _lhsOusedFields :: (Seq Identifier)
                           _lhsOusedLocals :: ([Identifier])
-                          _hdOallfields :: ([(Name,Type,Bool)])
-                          _hdOallnts :: ([Name])
+                          _hdOallfields :: ([(Identifier,Type,Bool)])
+                          _hdOallnts :: ([Identifier])
                           _hdOattrs :: ([(Identifier,Identifier)])
                           _hdOcon :: Identifier
                           _hdOfieldnames :: ([Identifier])
                           _hdOnt :: Identifier
-                          _tlOallfields :: ([(Name,Type,Bool)])
-                          _tlOallnts :: ([Name])
+                          _tlOallfields :: ([(Identifier,Type,Bool)])
+                          _tlOallnts :: ([Identifier])
                           _tlOattrs :: ([(Identifier,Identifier)])
                           _tlOcon :: Identifier
                           _tlOfieldnames :: ([Identifier])
@@ -377,19 +388,19 @@ sem_HsTokens_Cons (T_HsToken hd_) (T_HsTokens tl_) =
                           _tlIusedAttrs :: ([(Identifier,Identifier)])
                           _tlIusedFields :: (Seq Identifier)
                           _tlIusedLocals :: ([Identifier])
-                          -- "SemHsTokens.ag"(line 109, column 10)
+                          -- "SemHsTokens.ag"(line 108, column 10)
                           _lhsOtks =
                               _hdItok : _tlItks
-                          -- use rule "SemHsTokens.ag"(line 42, column 37)
+                          -- use rule "SemHsTokens.ag"(line 41, column 37)
                           _lhsOerrors =
                               _hdIerrors Seq.<> _tlIerrors
-                          -- use rule "SemHsTokens.ag"(line 84, column 40)
+                          -- use rule "SemHsTokens.ag"(line 83, column 40)
                           _lhsOusedAttrs =
                               _hdIusedAttrs ++ _tlIusedAttrs
-                          -- use rule "SemHsTokens.ag"(line 92, column 40)
+                          -- use rule "SemHsTokens.ag"(line 91, column 40)
                           _lhsOusedFields =
                               _hdIusedFields Seq.<> _tlIusedFields
-                          -- use rule "SemHsTokens.ag"(line 83, column 40)
+                          -- use rule "SemHsTokens.ag"(line 82, column 40)
                           _lhsOusedLocals =
                               _hdIusedLocals ++ _tlIusedLocals
                           -- copy rule (down)
@@ -446,19 +457,19 @@ sem_HsTokens_Nil  =
                           _lhsOusedAttrs :: ([(Identifier,Identifier)])
                           _lhsOusedFields :: (Seq Identifier)
                           _lhsOusedLocals :: ([Identifier])
-                          -- "SemHsTokens.ag"(line 110, column 10)
+                          -- "SemHsTokens.ag"(line 109, column 10)
                           _lhsOtks =
                               []
-                          -- use rule "SemHsTokens.ag"(line 42, column 37)
+                          -- use rule "SemHsTokens.ag"(line 41, column 37)
                           _lhsOerrors =
                               Seq.empty
-                          -- use rule "SemHsTokens.ag"(line 84, column 40)
+                          -- use rule "SemHsTokens.ag"(line 83, column 40)
                           _lhsOusedAttrs =
                               []
-                          -- use rule "SemHsTokens.ag"(line 92, column 40)
+                          -- use rule "SemHsTokens.ag"(line 91, column 40)
                           _lhsOusedFields =
                               Seq.empty
-                          -- use rule "SemHsTokens.ag"(line 83, column 40)
+                          -- use rule "SemHsTokens.ag"(line 82, column 40)
                           _lhsOusedLocals =
                               []
                       in  ( _lhsOerrors,_lhsOtks,_lhsOusedAttrs,_lhsOusedFields,_lhsOusedLocals))))
@@ -466,8 +477,8 @@ sem_HsTokens_Nil  =
 {-
    visit 0:
       inherited attributes:
-         allfields            : [(Name,Type,Bool)]
-         allnts               : [Name]
+         allfields            : [(Identifier,Type,Bool)]
+         allnts               : [Identifier]
          attrs                : [(Identifier,Identifier)]
          con                  : Identifier
          nt                   : Identifier
@@ -487,13 +498,13 @@ sem_HsTokensRoot :: HsTokensRoot ->
 sem_HsTokensRoot (HsTokensRoot _tokens) =
     (sem_HsTokensRoot_HsTokensRoot (sem_HsTokens _tokens))
 -- semantic domain
-newtype T_HsTokensRoot = T_HsTokensRoot (([(Name,Type,Bool)]) ->
-                                         ([Name]) ->
+newtype T_HsTokensRoot = T_HsTokensRoot (([(Identifier,Type,Bool)]) ->
+                                         ([Identifier]) ->
                                          ([(Identifier,Identifier)]) ->
                                          Identifier ->
                                          Identifier ->
                                          ( (Seq Error),([String]),([(Identifier,Identifier)]),([Identifier]),([Identifier])))
-data Inh_HsTokensRoot = Inh_HsTokensRoot {allfields_Inh_HsTokensRoot :: [(Name,Type,Bool)],allnts_Inh_HsTokensRoot :: [Name],attrs_Inh_HsTokensRoot :: [(Identifier,Identifier)],con_Inh_HsTokensRoot :: Identifier,nt_Inh_HsTokensRoot :: Identifier}
+data Inh_HsTokensRoot = Inh_HsTokensRoot {allfields_Inh_HsTokensRoot :: [(Identifier,Type,Bool)],allnts_Inh_HsTokensRoot :: [Identifier],attrs_Inh_HsTokensRoot :: [(Identifier,Identifier)],con_Inh_HsTokensRoot :: Identifier,nt_Inh_HsTokensRoot :: Identifier}
 data Syn_HsTokensRoot = Syn_HsTokensRoot {errors_Syn_HsTokensRoot :: Seq Error,textLines_Syn_HsTokensRoot :: [String],usedAttrs_Syn_HsTokensRoot :: [(Identifier,Identifier)],usedFields_Syn_HsTokensRoot :: [Identifier],usedLocals_Syn_HsTokensRoot :: [Identifier]}
 wrap_HsTokensRoot (T_HsTokensRoot sem) (Inh_HsTokensRoot _lhsIallfields _lhsIallnts _lhsIattrs _lhsIcon _lhsInt) =
     (let ( _lhsOerrors,_lhsOtextLines,_lhsOusedAttrs,_lhsOusedFields,_lhsOusedLocals) =
@@ -513,8 +524,8 @@ sem_HsTokensRoot_HsTokensRoot (T_HsTokens tokens_) =
                               _lhsOerrors :: (Seq Error)
                               _lhsOusedAttrs :: ([(Identifier,Identifier)])
                               _lhsOusedLocals :: ([Identifier])
-                              _tokensOallfields :: ([(Name,Type,Bool)])
-                              _tokensOallnts :: ([Name])
+                              _tokensOallfields :: ([(Identifier,Type,Bool)])
+                              _tokensOallnts :: ([Identifier])
                               _tokensOattrs :: ([(Identifier,Identifier)])
                               _tokensOcon :: Identifier
                               _tokensOnt :: Identifier
@@ -523,13 +534,13 @@ sem_HsTokensRoot_HsTokensRoot (T_HsTokens tokens_) =
                               _tokensIusedAttrs :: ([(Identifier,Identifier)])
                               _tokensIusedFields :: (Seq Identifier)
                               _tokensIusedLocals :: ([Identifier])
-                              -- "SemHsTokens.ag"(line 37, column 18)
+                              -- "SemHsTokens.ag"(line 36, column 18)
                               _tokensOfieldnames =
                                   map (\(n,_,_) -> n) _lhsIallfields
-                              -- "SemHsTokens.ag"(line 99, column 18)
+                              -- "SemHsTokens.ag"(line 98, column 18)
                               _lhsOusedFields =
                                   Seq.toList _tokensIusedFields
-                              -- "SemHsTokens.ag"(line 106, column 18)
+                              -- "SemHsTokens.ag"(line 105, column 18)
                               _lhsOtextLines =
                                   showTokens _tokensItks
                               -- use rule "SemHsTokens.ag"(line 16, column 18)
