@@ -6,7 +6,8 @@ import List                          (isSuffixOf)
 import Monad                         (zipWithM_)
 import Data.Maybe
 
-import qualified Data.Map as Map (elems, partitionWithKey, unionWith)
+import qualified Data.Set as Set
+import qualified Data.Map as Map
 import qualified UU.DData.Seq as Seq ((<>),toList)
 import Pretty
 
@@ -21,15 +22,18 @@ import qualified Order              as Pass3 (sem_Grammar,  wrap_Grammar,  Syn_G
 import qualified GenerateCode       as Pass4 (sem_CGrammar, wrap_CGrammar, Syn_CGrammar(..), Inh_CGrammar(..))
 import qualified PrintCode          as Pass5 (sem_Program,  wrap_Program,  Syn_Program (..), Inh_Program (..))
 import qualified PrintErrorMessages as PrErr (sem_Errors ,  wrap_Errors ,  Syn_Errors  (..), Inh_Errors  (..), isError)
+import qualified TfmToVisage        as PassV (sem_Grammar,  wrap_Grammar,  Syn_Grammar (..), Inh_Grammar (..))
 
 import qualified AbstractSyntaxDump as GrammarDump (sem_Grammar,  wrap_Grammar,  Syn_Grammar (..), Inh_Grammar (..))
 import qualified CodeSyntaxDump as CGrammarDump (sem_CGrammar,  wrap_CGrammar,  Syn_CGrammar (..), Inh_CGrammar (..))
+import qualified Visage as VisageDump (sem_VisageGrammar, wrap_VisageGrammar, Syn_VisageGrammar(..), Inh_VisageGrammar(..))
 
 import Options
 import Version       (banner)
 import Parser        (parseAG, depsAG)
 import ErrorMessages (Error(ParserError), Errors)
-import CommonTypes   (Blocks)
+import CommonTypes
+import ATermWrite
 
 
 main :: IO ()
@@ -52,14 +56,17 @@ main
 compile :: Options -> String -> String -> IO ()
 compile flags input output
  = do (output0,parseErrors) <- parseAG (searchPath flags) (inputFile input)
+      irrefutableMap <- readIrrefutableMap flags
 
       let output1   = Pass1.wrap_AG              (Pass1.sem_AG                                 output0 ) Pass1.Inh_AG       {Pass1.options_Inh_AG       = flags}
           flags'    = Pass1.pragmas_Syn_AG       output1 $ flags
           grammar1  = Pass1.output_Syn_AG        output1
-          output1a  = Pass1a.wrap_Grammar        (Pass1a.sem_Grammar grammar1                          ) Pass1a.Inh_Grammar {Pass1a.options_Inh_Grammar = flags'}
+          output1a  = Pass1a.wrap_Grammar        (Pass1a.sem_Grammar grammar1                          ) Pass1a.Inh_Grammar {Pass1a.options_Inh_Grammar = flags', Pass1a.forcedIrrefutables_Inh_Grammar = irrefutableMap }
           grammar1a =Pass1a.output_Syn_Grammar   output1a
           output2   = Pass2.wrap_Grammar         (Pass2.sem_Grammar grammar1a                          ) Pass2.Inh_Grammar  {Pass2.options_Inh_Grammar  = flags'}
           grammar2  = Pass2.output_Syn_Grammar   output2
+          outputV   = PassV.wrap_Grammar         (PassV.sem_Grammar grammar2                           ) PassV.Inh_Grammar  {}
+          grammarV  = PassV.visage_Syn_Grammar   outputV
           output3   = Pass3.wrap_Grammar         (Pass3.sem_Grammar grammar2                           ) Pass3.Inh_Grammar  {Pass3.options_Inh_Grammar  = flags'}
           grammar3  = Pass3.output_Syn_Grammar   output3
           output4   = Pass4.wrap_CGrammar        (Pass4.sem_CGrammar(Pass3.output_Syn_Grammar  output3)) Pass4.Inh_CGrammar {Pass4.options_Inh_CGrammar = flags'}
@@ -69,6 +76,9 @@ compile flags input output
           dump1    = GrammarDump.wrap_Grammar   (GrammarDump.sem_Grammar grammar1                     ) GrammarDump.Inh_Grammar
           dump2    = GrammarDump.wrap_Grammar   (GrammarDump.sem_Grammar grammar2                     ) GrammarDump.Inh_Grammar
           dump3    = CGrammarDump.wrap_CGrammar (CGrammarDump.sem_CGrammar grammar3                   ) CGrammarDump.Inh_CGrammar
+          
+          outputVisage = VisageDump.wrap_VisageGrammar (VisageDump.sem_VisageGrammar grammarV) VisageDump.Inh_VisageGrammar
+          aterm        = VisageDump.aterm_Syn_VisageGrammar outputVisage
 
           parseErrorList   = map message2error parseErrors
           errorList        = parseErrorList
@@ -79,12 +89,12 @@ compile flags input output
                                            Seq.<> Pass4.errors_Syn_CGrammar output4
                                            )
                                            
-          fatalErrorList = filter PrErr.isError errorList
+          fatalErrorList = filter (PrErr.isError flags') errorList
           
           allErrors = if null parseErrors
                       then if wignore flags'
                            then fatalErrorList
-                           else errorsToFront errorList
+                           else errorsToFront flags' errorList
                       else take 1 parseErrorList
                       -- the other 1000 or so parse errors are usually not that informative
                       
@@ -98,9 +108,9 @@ compile flags input output
           (pragmaBlocks, blocks2)    = Map.partitionWithKey (\k _->k=="optpragmas") blocks1
           (importBlocks, textBlocks) = Map.partitionWithKey (\k _->k=="imports"   ) blocks2
           
-          importBlocksTxt = vlist_sep "" . map addLocationPragma . Map.elems $ importBlocks
-          textBlocksDoc   = vlist_sep "" . map addLocationPragma . Map.elems $ textBlocks
-          pragmaBlocksTxt = unlines . concat . map fst      . Map.elems $ pragmaBlocks
+          importBlocksTxt = vlist_sep "" . map addLocationPragma . concat . Map.elems $ importBlocks
+          textBlocksDoc   = vlist_sep "" . map addLocationPragma . concat . Map.elems $ textBlocks
+          pragmaBlocksTxt = unlines . concat . map fst      . concat . Map.elems $ pragmaBlocks
           
           outputfile = if null output then outputFile input else output
           
@@ -120,10 +130,10 @@ compile flags input output
           mainName = stripPath $ defaultModuleName input
           mainFile = defaultModuleName input
 
-          nrOfErrorsToReport = length $ filter PrErr.isError errorsToReport
-          nrOfWarningsToReport = length $ filter (not.PrErr.isError) errorsToReport
-          totalNrOfErrors = length $ filter PrErr.isError allErrors
-          totalNrOfWarnings = length $ filter (not.PrErr.isError) allErrors
+          nrOfErrorsToReport = length $ filter (PrErr.isError flags') errorsToReport
+          nrOfWarningsToReport = length $ filter (not.(PrErr.isError flags')) errorsToReport
+          totalNrOfErrors = length $ filter (PrErr.isError flags') allErrors
+          totalNrOfWarnings = length $ filter (not.(PrErr.isError flags')) allErrors
           additionalErrors = totalNrOfErrors - nrOfErrorsToReport
           additionalWarnings = totalNrOfWarnings - nrOfWarningsToReport
           pluralS n = if n == 1 then "" else "s"
@@ -141,15 +151,25 @@ compile flags input output
            
       if not (null fatalErrorList) 
        then exitFailure
-       else if sepSemMods flags'
+       else 
+        do if genvisage flags'
+            then writeFile (outputfile++".visage") (writeATerm aterm)
+            else return ()
+            
+           if genAttributeList flags'
+            then writeAttributeList (outputfile++".attrs") (Pass1a.allAttributes_Syn_Grammar output1a)
+            else return ()
+
+           if sepSemMods flags'
             then do -- alternative module gen
                     Pass5.genIO_Syn_Program output5
                     if not (null errorsToStopOn) then exitFailure else return ()
             else do -- conventional module gen
                     let doc = vlist [ pp optionsLine
+                                    , pp pragmaBlocksTxt
                                     , pp $ take 70 ("-- UUAGC " ++ drop 50 banner ++ " (" ++ input) ++ ")"
                                     , pp $ if isNothing $ Pass1.moduleDecl_Syn_AG output1
-                                           then moduleHeader flags' input
+                                           then moduleHeader flags' mainName
                                            else mkModuleHeader (Pass1.moduleDecl_Syn_AG output1) mainName "" "" False
                                     , pp importBlocksTxt
                                     , textBlocksDoc
@@ -191,8 +211,8 @@ message2error (Msg expect pos action) = ParserError pos (show expect) actionStri
 
               Other ms -> ms
 
-errorsToFront :: [Error] -> [Error]
-errorsToFront mesgs = filter PrErr.isError mesgs ++ filter (not.PrErr.isError) mesgs
+errorsToFront :: Options -> [Error] -> [Error]
+errorsToFront flags mesgs = filter (PrErr.isError flags) mesgs ++ filter (not.(PrErr.isError flags)) mesgs
 
 
 moduleHeader :: Options -> String -> String
@@ -222,13 +242,8 @@ defaultModuleName name
    else name
 
 stripPath :: String -> String
-stripPath s
-  = stripPath' s ""
-
-stripPath' [] acc = acc
-stripPath' (x:xs) acc
-  | x == '/' || x == '\\' = stripPath' xs ""
-  | otherwise = stripPath' xs (acc ++ [x])
+stripPath
+  = reverse . takeWhile (\c -> c /= '/' && c /= '\\') . reverse
 
 mkMainName :: String -> Maybe (String, String,String) -> String
 mkMainName defaultName Nothing
@@ -264,3 +279,20 @@ reportDeps flags files
     combine (fs, mesgs) (fsr, mesgsr)
       = (fs ++ fsr, mesgs ++ mesgsr)
 
+
+writeAttributeList :: String -> AttrMap -> IO ()
+writeAttributeList file mp
+  = writeFile file s
+  where
+    s = show $ map (\(x,y) -> (show x, y)) $ Map.toList $ Map.map (map (\(x,y) -> (show x, y)) . Map.toList . Map.map (map (\(x,y) -> (show x, show y)) . Set.toList)) $ mp
+
+readIrrefutableMap :: Options -> IO AttrMap
+readIrrefutableMap flags
+  = case forceIrrefutables flags of
+      Just file -> do s <- readFile file
+                      seq (length s) (return ())
+                      let lists :: [(String,[(String,[(String, String)])])]
+                          lists = read s
+                      return $ Map.fromList [ (identifier n, Map.fromList [(identifier c, Set.fromList [ (identifier fld, identifier attr) | (fld,attr) <- ss ]) | (c,ss) <- cs ]) | (n,cs) <- lists ]
+      Nothing   -> return Map.empty
+
