@@ -1,6 +1,5 @@
 module CommonTypes where
 
---import UU.Pretty (PP,text,pp)
 import Pretty
 import UU.Scanner.Position(Pos,noPos)
 import qualified Data.Map as Map
@@ -9,7 +8,7 @@ import Data.Set(Set)
 import qualified Data.Set as Set
 
 
-type Blocks = Map String ([String], Pos)
+type Blocks = Map String [([String], Pos)]
 
 data Identifier  = Ident {getName::String , getPos::Pos}
 
@@ -26,7 +25,7 @@ instance PP Identifier where
   pp = text . getName
 
 data Type = Haskell String
-          | NT Name
+          | NT Identifier [String]
 
 data ComplexType = List Type
                  | Tuple [(Identifier, Type)]
@@ -38,30 +37,27 @@ instance Show ComplexType where
   show (Maybe t ) = "Maybe " ++ show t
 
 instance Show Type where
-  show (Haskell t) = t
-  show (NT nt  ) = getName nt
+  show = typeToHaskellString Nothing []
 
-instance Eq Type where
-  NT x == NT y = x == y
-  _      == _      = False
-  
-type Attributes  = Map Name Type
-type TypeSyns    = [(Nonterminal,ComplexType)]
-
-type AttrNames   = [(Name,Type,(String,String,String))]
-type UseMap      = Map Nonterminal (Map Name (String,String,String))
-type PragmaMap   = Map Nonterminal (Map Constructor (Set Name))
-type Fields      = [(Name,Type)]
-type Derivings   = Map Nonterminal (Set Name)
+type Attributes  = Map Identifier Type
+type TypeSyns    = [(NontermIdent,ComplexType)]
+type ParamMap    = Map NontermIdent [Identifier]
+type AttrNames   = [(Identifier,Type,(String,String,String))]
+type UseMap      = Map NontermIdent (Map Identifier (String,String,String))
+type PragmaMap   = Map NontermIdent (Map ConstructorIdent (Set Identifier))
+type AttrMap     = Map NontermIdent (Map ConstructorIdent (Set (Identifier,Identifier)))
+type Fields      = [(Identifier,Type)]
+type Derivings   = Map NontermIdent (Set Identifier)
+type ClassContext = [(Identifier, [String])]
+type ContextMap  = Map NontermIdent ClassContext
 type Strings     = [String]
-type Name        = Identifier
-type Nonterminal = Name
-type Constructor = Name
-type AttrOrderMap = Map Nonterminal (Map Constructor (Set Dependency))
+type NontermIdent     = Identifier
+type ConstructorIdent = Identifier
+type AttrOrderMap = Map NontermIdent (Map ConstructorIdent (Set Dependency))
 data Dependency = Dependency (Identifier,Identifier) (Identifier,Identifier) deriving (Eq,Ord,Show)
 
-type AttrEnv = ( [Name]
-               , [(Name,Name)]
+type AttrEnv = ( [Identifier]
+               , [(Identifier,Identifier)]
                )
 
 identifier x   = Ident x noPos               
@@ -75,23 +71,23 @@ _FIELD = identifier "field"
 _FIRST = identifier "first"
 _LAST  = identifier "last"
 
-sdtype :: Nonterminal -> String
+sdtype :: NontermIdent -> String
 sdtype nt = "T_"++getName nt
 
-cataname ::  String -> Name -> String
+cataname ::  String -> Identifier -> String
 cataname pre name = pre++getName name
 
-conname :: Bool -> Nonterminal -> Constructor -> String
+conname :: Bool -> NontermIdent -> ConstructorIdent -> String
 conname rename nt con | rename =  getName nt ++ "_" ++ getName con
                       | otherwise = getName con
 
-semname  ::  String -> Nonterminal -> Constructor -> String
+semname  ::  String -> NontermIdent -> ConstructorIdent -> String
 semname pre nt con =  pre ++ (getName nt ++ "_" ++ getName con)
 
-lhsname :: Bool -> Name -> String
+lhsname :: Bool -> Identifier -> String
 lhsname isIn = attrname isIn _LHS
 
-attrname :: Bool -> Name -> Name -> String
+attrname :: Bool -> Identifier -> Identifier -> String
 attrname isIn field attr | field == _LOC   = locname attr 
                          | field == _INST  = instname attr
                          | field == _INST' = inst'name attr
@@ -105,25 +101,47 @@ instname v  = getName v ++ "_val_"
 inst'name v = getName v ++ "_"
 fieldname v =  getName v++"_"
 
-typeToString :: Nonterminal -> Type -> String
-typeToString _ (Haskell t)  = t
-typeToString nt (NT t   ) | t == _SELF = getName nt
-                          | otherwise  = getName t
+typeToAGString :: Type -> String
+typeToAGString tp
+  = case tp of
+      Haskell t -> t
+      NT nt tps -> formatNonterminalToHaskell (getName nt) (map (\s -> "{" ++ s ++ "}") tps)
+
+typeToHaskellString :: Maybe NontermIdent -> [String] -> Type -> String
+typeToHaskellString mbNt params tp
+  = case tp of
+      Haskell t -> t
+      NT nt tps | nt == _SELF -> formatNonterminalToHaskell (maybe "Unknown" getName mbNt) params
+                | otherwise   -> formatNonterminalToHaskell (getName nt) tps
+
+formatNonterminalToHaskell :: String -> [String] -> String
+formatNonterminalToHaskell nt tps
+  = unwords (nt:tps)
 
 ind :: String -> String
 ind s = replicate 3 ' ' ++ s
 
-_NOCASE :: Name
+_NOCASE :: Identifier
 _NOCASE = identifier "nocase"
 
-hasPragma :: PragmaMap -> Nonterminal -> Constructor -> Name -> Bool
+hasPragma :: PragmaMap -> NontermIdent -> ConstructorIdent -> Identifier -> Bool
 hasPragma mp nt con nm
   = nm `Set.member` Map.findWithDefault Set.empty con (Map.findWithDefault Map.empty nt mp)
   
 isNonterminal :: Type -> Bool
-isNonterminal (NT _) = True
-isNonterminal _      = False
+isNonterminal (NT _ _) = True
+isNonterminal _        = False
 
-extractNonterminal :: Type -> Nonterminal
-extractNonterminal (NT n) = n
+isSELFNonterminal :: Type -> Bool
+isSELFNonterminal (NT nt _) | nt == _SELF = True
+isSELFNonterminal _                       = False
+
+extractNonterminal :: Type -> NontermIdent
+extractNonterminal (NT n _) = n
+
+nontermArgs :: Type -> [String]
+nontermArgs tp
+  = case tp of
+      NT _ args -> args
+      _         -> [] 
 
