@@ -44,10 +44,14 @@ options     =  [ Option ['m']     []                (NoArg (moduleOpt Nothing))
                , Option []        ["dumpgrammar"]   (NoArg dumpgrammarOpt)      "Dump internal grammar representation (in generated code)"
                , Option []        ["dumpcgrammar"]  (NoArg dumpcgrammarOpt)      "Dump internal cgrammar representation (in generated code)"
                , Option []        ["gentraces"]     (NoArg genTracesOpt)        "Generate trace expressions (in generated code)"
+               , Option []        ["genusetraces"]  (NoArg genUseTracesOpt)     "Generate trace expressions at attribute use sites (in generated code)"
                , Option []        ["gencostcentres"] (NoArg genCostCentresOpt)  "Generate cost centre pragmas (in generated code)"
                , Option []        ["genlinepragmas"] (NoArg genLinePragmasOpt)  "Generate GHC LINE pragmas (in generated code)"
                , Option []        ["sepsemmods"]    (NoArg sepSemModsOpt)       "Generate separate modules for semantic functions (in generated code)"
                , Option ['M']     ["genfiledeps"] (NoArg genFileDepsOpt) "Generate a list of dependencies on the input AG files"
+               , Option []        ["genvisage"] (NoArg genVisageOpt)  "Generate output for the AG visualizer Visage"
+               , Option []        ["genattrlist"] (NoArg genAttrListOpt) "Generate a list of all explicitly defined attributes (outside irrefutable patterns)"
+               , Option []        ["forceirrefutable"] (OptArg forceIrrefutableOpt "file") "Force a set of explicitly defined attributes to be irrefutable, specify file containing the attribute set"
                ]
 
 allc = "dcfsprm"
@@ -90,10 +94,14 @@ data Options = Options{ moduleName :: ModuleHeader
                       , dumpgrammar :: Bool
                       , dumpcgrammar :: Bool
                       , genTraces :: Bool
+                      , genUseTraces :: Bool
                       , genCostCentres :: Bool
                       , sepSemMods :: Bool
                       , genFileDeps :: Bool
                       , genLinePragmas :: Bool
+                      , genvisage :: Bool
+                      , genAttributeList :: Bool
+                      , forceIrrefutables :: Maybe String
                       } deriving Show
 noOptions = Options { moduleName    = NoName
                     , dataTypes     = False
@@ -133,10 +141,14 @@ noOptions = Options { moduleName    = NoName
                     , dumpgrammar   = False
                     , dumpcgrammar  = False
                     , genTraces     = False
+                    , genUseTraces  = False
                     , genCostCentres = False
                     , sepSemMods     = False
                     , genFileDeps    = False
                     , genLinePragmas = False
+                    , genvisage      = False
+                    , genAttributeList = False
+                    , forceIrrefutables = Nothing
                     }
 
 
@@ -176,10 +188,14 @@ wmaxErrsOpt n   opts = opts{wmaxerrs     = read n}
 dumpgrammarOpt  opts = opts{dumpgrammar  = True}
 dumpcgrammarOpt opts = opts{dumpcgrammar = True}
 genTracesOpt    opts = opts{genTraces    = True}
+genUseTracesOpt opts = opts{genUseTraces = True}
 genCostCentresOpt opts = opts{genCostCentres = True}
 sepSemModsOpt opts = opts{sepSemMods = True}
 genFileDepsOpt opts = opts{genFileDeps = True}
 genLinePragmasOpt opts = opts{genLinePragmas = True}
+genVisageOpt opts = opts{genvisage = True }
+genAttrListOpt opts = opts { genAttributeList = True }
+forceIrrefutableOpt mbNm opts = opts { forceIrrefutables = mbNm }
 
 outputOpt  file  opts = opts{outputFiles  = file : outputFiles opts}            
 searchPathOpt  path  opts = opts{searchPath  = extract path ++ searchPath opts}            
