@@ -20,6 +20,7 @@ import UU.Scanner.GenTokenParser
 import UU.Scanner.Position
 import UU.Scanner.TokenShow()
 import System.Directory
+import HsTokenScanner
 
 
 type AGParser = AnaParser Input  Pair Token Pos
@@ -129,7 +130,7 @@ pNontSet = set0
             <|> All <$ pStar
             <|> pParens set0
 
-pNames :: AGParser [Name]
+pNames :: AGParser [Identifier]
 pNames = pList1 pIdentifier
 
 pAG  :: AGParser AG
@@ -138,27 +139,33 @@ pAG  = AG <$> pElems
 pElems :: AGParser Elems
 pElems = pList_ng pElem
 
-pComplexType =  List <$> pBracks pType 
+pComplexType =  List <$> pBracks pTypeEncapsulated 
             <|> Maybe <$ pMAYBE <*> pType
             <|> tuple <$> pParens (pListSep pComma field)
- where field = (,) <$> ((Just <$> pIdentifier <* pColon) `opt` Nothing) <*> pType
+ where field = (,) <$> ((Just <$> pIdentifier <* pColon) `opt` Nothing) <*> pTypeEncapsulated
        tuple xs = Tuple [(fromMaybe (Ident ("x"++show n) noPos) f, t) 
                         | (n,(f,t)) <- zip [1..] xs
                         ]
 pElem :: AGParser Elem
 pElem =  Data <$> pDATA
+              <*> pOptClassContext
               <*> pNontSet
+              <*> pList pIdentifier
               <*> pOptAttrs
               <*> pAlts
              <*> pSucceed False
      <|> Attr <$> pATTR
+              <*> pOptClassContext
               <*> pNontSet
               <*> pAttrs
      <|> Type <$> pTYPE
+              <*> pOptClassContext
               <*> pIdentifierU
+              <*> pList pIdentifier
               <*  pEquals
               <*> pComplexType
      <|> Sem  <$> pSEM
+              <*> pOptClassContext
               <*> pNontSet
               <*> pOptAttrs
               <*> pSemAlts
@@ -185,11 +192,21 @@ pElem =  Data <$> pDATA
      <|> codeBlock <$> (pIdentifier <|> pSucceed (Ident "" noPos)) <*> pCodeBlock <?> "a statement"
            where codeBlock nm (txt,pos) = Txt pos nm (lines txt)
 
+
 -- Insertion is expensive for pCodeBlock in order to prevent infinite inserts.
 pCodeBlock ::  AGParser (String,Pos)
 pCodeBlock   = pCostValToken 90 TkTextln "" <?> "a code block"
 
 
+pOptClassContext :: AGParser ClassContext
+pOptClassContext
+  =   pClassContext <* pDoubleArrow
+  <|> pSucceed []
+
+pClassContext :: AGParser ClassContext
+pClassContext
+  = pListSep pComma ((,) <$> pIdentifierU <*> pList pTypeHaskellAnyAsString)
+
 pAttrs :: AGParser Attrs
 pAttrs = Attrs <$> pOBrackPos <*> (concat <$> pList pInhAttrNames <?> "inherited attribute declarations")
                               <* pBar    <*> (concat <$> pList pAttrNames <?> "chained attribute declarations"  )
@@ -199,8 +216,27 @@ pAttrs = Attrs <$> pOBrackPos <*> (concat <$> pList pInhAttrNames <?> "inherited
 pOptAttrs :: AGParser Attrs
 pOptAttrs = pAttrs `opt` Attrs noPos [] [] []
 
+pTypeNt :: AGParser Type
+pTypeNt
+  =   (\nt -> NT nt []) <$> pIdentifierU
+  <|> pParens (NT <$> pIdentifierU <*> pList pTypeHaskellAnyAsString)
+
+pTypeHaskellAnyAsString :: AGParser String
+pTypeHaskellAnyAsString
+  =   getName <$> pIdentifier
+  <|> getName <$> pIdentifierU
+  <|> pCodescrap' <?> "a type"
+
+-- if the type is within some kind of parentheses or brackets (then we allow lowercase identifiers as well)
+pTypeEncapsulated :: AGParser Type
+pTypeEncapsulated
+  =   pParens pTypeEncapsulated
+  <|> NT <$> pIdentifierU <*> pList pTypeHaskellAnyAsString
+  <|> (Haskell . getName) <$> pIdentifier
+  <|> Haskell <$> pCodescrap'  <?> "a type"
+
 pType :: AGParser Type
-pType =  NT <$> pIdentifierU
+pType =  pTypeNt
      <|> Haskell <$> pCodescrap'  <?> "a type"
 
 
@@ -231,7 +267,7 @@ pFields    = concat <$> pList_ng pField <?> "fields"
 pField     :: AGParser Fields
 pField     =  (\nms tp -> map (flip (,) tp) nms)
            <$> pIdentifiers <* pColon <*> pType
-           <|> (\s -> [(Ident (mklower (getName s)) (getPos s) ,NT s)]) <$> pIdentifierU
+           <|> (\s -> [(Ident (mklower (getName s)) (getPos s) ,NT s [])]) <$> pIdentifierU
 
 mklower :: String -> String
 mklower (x:xs) = toLower x : xs
@@ -256,8 +292,9 @@ pConstructorSet =  pChainl (CDifference <$ pMinus) term2
 pSemAlts :: AGParser SemAlts
 pSemAlts =  pList pSemAlt <?> "SEM alternatives"
 
-pFieldIdentifier =  pIdentifier 
-                <|> Ident "lhs"  <$> pLHS 
+pFieldIdentifier :: AGParser Identifier
+pFieldIdentifier =  pIdentifier
+                <|> Ident "lhs"  <$> pLHS
                 <|> Ident "loc"  <$> pLOC
                 <|> Ident "inst" <$> pINST
 
@@ -266,16 +303,16 @@ pSemDef = (\x fs -> map ($ x) fs)<$> pFieldIdentifier <*> pList1 pAttrDef
       <|>                            pLOC              *> pList1 pLocDecl
       <|>                            pINST             *> pList1 pInstDecl
       <|>  pSEMPRAGMA *> pList1 (SemPragma <$> pNames)
-      <|> (\a b -> [AttrOrderBefore a b]) <$> pAttr <* pSmaller <*> pAttr
+      <|> (\a b -> [AttrOrderBefore a [b]]) <$> pList1 pAttr <* pSmaller <*> pAttr
       <|> (\pat owrt exp -> [Def (pat ()) exp owrt]) <$> pPattern (const <$> pAttr) <*> pAssign <*> pExpr
  
 pAttr = (,) <$> pFieldIdentifier <* pDot <*> pIdentifier
  
-pAttrDef :: AGParser (Name -> SemDef)
+pAttrDef :: AGParser (Identifier -> SemDef)
 pAttrDef = (\pat owrt exp fld -> Def (pat fld) exp owrt)
            <$ pDot <*> pattern <*> pAssign <*> pExpr
   where pattern =  pPattern pVar
-               <|> (\a fld -> Alias fld a (Underscore noPos) []) <$> pIdentifier
+               <|> (\ir a fld -> ir $ Alias fld a (Underscore noPos) []) <$> ((Irrefutable <$ pTilde) `opt` id) <*> pIdentifier
 
 
 nl2sp :: Char -> Char
@@ -292,34 +329,34 @@ pLocType = (Haskell . getName) <$> pIdentifierU
 
 pInstDecl :: AGParser SemDef
 pInstDecl = (\ident tp -> TypeDef ident tp)
-             <$ pDot <*> pIdentifier <* pColon <*> (NT <$> pIdentifierU)
+             <$ pDot <*> pIdentifier <* pColon <*> pTypeNt
 
 pSemDefs :: AGParser SemDefs
 pSemDefs =  concat <$> pList_ng pSemDef  <?> "attribute rules"
 
-pVar :: AGParser (Name -> (Name, Name))
+pVar :: AGParser (Identifier -> (Identifier, Identifier))
 pVar = (\att fld -> (fld,att)) <$> pIdentifier
 
  
 pExpr :: AGParser Expression
-pExpr = (\(str,pos) ->  Expression pos str) <$> ( pCodescrapL  ) <?> "an expression"
+pExpr = (\(str,pos) ->  Expression pos (lexTokens pos str)) <$> pCodescrapL <?> "an expression"
 
 pAssign :: AGParser Bool
 pAssign =  False <$ pReserved "="
        <|> True  <$ pReserved ":="
        
-pAttrDefs :: AGParser (Name -> [SemDef])
+pAttrDefs :: AGParser (Identifier -> [SemDef])
 pAttrDefs = (\fs field -> map ($ field) fs) <$> pList1 pAttrDef  <?> "attribute definitions"
 
-pPattern :: AGParser (a -> (Name,Name)) -> AGParser (a -> Pattern)
+pPattern :: AGParser (a -> (Identifier,Identifier)) -> AGParser (a -> Pattern)
 pPattern pvar = pPattern2 where
   pPattern0 =  (\i pats a -> Constr i (map ($ a) pats))
                <$> pIdentifierU <*> pList  pPattern1
                <|> pPattern1 <?> "a pattern"
-  pPattern1 =  pvariable 
+  pPattern1 =  pvariable
            <|> pPattern2
-  pvariable = (\var pat a -> case var a of (fld,att) -> Alias fld att (pat a) []) 
-           <$> pvar <*> ((pAt *> pPattern1) `opt` const (Underscore noPos)) 
+  pvariable = (\ir var pat a -> case var a of (fld,att) -> ir $ Alias fld att (pat a) []) 
+           <$> ((Irrefutable <$ pTilde) `opt` id) <*> pvar <*> ((pAt *> pPattern1) `opt` const (Underscore noPos)) 
   pPattern2 = (mkTuple <$> pOParenPos <*> pListSep pComma pPattern0 <* pCParen )
           <|> (const . Underscore) <$> pUScore <?> "a pattern"
     where mkTuple _ [x] a = x a
@@ -333,10 +370,10 @@ pCodescrap' = fst <$> pCodescrap
 pCodescrap ::  AGParser (String,Pos)
 pCodescrap   = pCodeBlock
 
-pSEM, pATTR, pDATA, pUSE, pLOC,pINCLUDE, pTYPE, pEquals, pColonEquals,
-      pBar, pColon, pLHS,pINST,pSET,pDERIVING,pMinus,pIntersect,pArrow,
-      pDot, pUScore, pEXT,pAt,pStar, pSmaller, pWRAPPER, pMAYBE, pMODULE
-      :: AGParser  Pos
+pSEM, pATTR, pDATA, pUSE, pLOC,pINCLUDE, pTYPE, pEquals, pColonEquals, pTilde,
+      pBar, pColon, pLHS,pINST,pSET,pDERIVING,pMinus,pIntersect,pDoubleArrow,pArrow,
+      pDot, pUScore, pEXT,pAt,pStar, pSmaller, pWRAPPER, pPRAGMA, pMAYBE, pMODULE
+      :: AGParser Pos
 pSET         = pCostReserved 90 "SET"     <?> "SET"
 pDERIVING    = pCostReserved 90 "DERIVING"<?> "DERIVING"
 pWRAPPER     = pCostReserved 90 "WRAPPER" <?> "WRAPPER"
@@ -359,9 +396,11 @@ pUScore      = pCostReserved 5  "_"       <?> "_"
 pColon       = pCostReserved 5  ":"       <?> ":"
 pEquals      = pCostReserved 5  "="       <?> "="
 pColonEquals = pCostReserved 5  ":="      <?> ":="
+pTilde       = pCostReserved 5  "~"       <?> "~"
 pBar         = pCostReserved 5  "|"       <?> "|"
 pIntersect   = pCostReserved 5  "/\\"     <?> "/\\"
 pMinus       = pCostReserved 5  "-"       <?> "-"
+pDoubleArrow = pCostReserved 5  "=>"      <?> "=>"
 pArrow       = pCostReserved 5  "->"      <?> "->"
 pStar        = pCostReserved 5  "*"       <?> "*"
 pSmaller     = pCostReserved 5  "<"       <?> "<"
