@@ -51,6 +51,7 @@ scan p xs = Just (scan' xs)
         scan' ('@' :rs)          = (reserved "@" p, advc 1 p, rs)
         scan' (',' :rs)          = (reserved "," p, advc 1 p, rs)
         scan' ('_' :rs)          = (reserved "_" p, advc 1 p, rs)
+        scan' ('~' :rs)          = (reserved "~" p, advc 1 p, rs)
         scan' ('<' :rs)          = (reserved "<" p, advc 1 p, rs)
         scan' ('[' :rs)          = (reserved "[" p, advc 1 p, rs)
         scan' (']' :rs)          = (reserved "]" p, advc 1 p, rs)
@@ -66,6 +67,7 @@ scan p xs = Just (scan' xs)
                                                , advc (1+length str) p,rest)
                                           else (valueToken TkString str p, advc (2+length str) p, tail rest)
 
+        scan' ('=' : '>' : rs)   = (reserved "=>" p, advc 2 p, rs)
         scan' ('=' :rs)          = (reserved "=" p, advc 1 p, rs)
         scan' (':':'=':rs)       = (reserved ":=" p, advc 2 p, rs)
 
@@ -111,7 +113,7 @@ scanBeginOfLine p xs
 
 ident = span isValid
  where isValid x = isAlphaNum x || x =='_' || x == '\''
-keywords = [ "DATA", "EXT", "ATTR", "SEM","TYPE", "USE", "loc","lhs", "inst", "first", "last", "INCLUDE"
+keywords = [ "DATA", "EXT", "ATTR", "SEM","TYPE", "USE", "loc","lhs", "inst", "INCLUDE"
            , "SET","DERIVING","FOR", "WRAPPER", "MAYBE"
            , "PRAGMA", "SEMPRAGMA", "MODULE"
            ]
